from six import print_

from django.core.management.base import BaseCommand

from ...base import get_registered_layout_uids, get_registered_plugin_uids
from ...models import DashboardEntry

__title__ = 'dash.management.commands.dash_find_broken_dashboard_entries'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2017 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('Command',)


class Command(BaseCommand):
    """Add the missing plugins to database (``dash.models.DashboardPlugin``).

    This command shall be ran every time a developer adds a new plugin.
    """

    def handle(self, *args, **options):

        dashboard_entries = DashboardEntry._default_manager \
            .all() \
            .only('id', 'plugin_uid', 'layout_uid') \
            .values_list('id', 'plugin_uid', 'layout_uid')

        broken_plugin_entries = []
        broken_layout_entries = []

        registered_plugin_uids = get_registered_plugin_uids()
        registered_layout_uids = get_registered_layout_uids()

        for entry_id, plugin_uid, layout_uid in dashboard_entries:
            if plugin_uid not in registered_plugin_uids:
                broken_plugin_entries.append((entry_id, plugin_uid))

            if layout_uid not in registered_layout_uids:
                broken_layout_entries.append((entry_id, layout_uid))

        if broken_plugin_entries:
            print_("Broken plugin entries found!", broken_plugin_entries)

        if broken_layout_entries:
            print_("Broken layout entries found!", broken_layout_entries)



from django.utils.translation import ugettext_lazy as _

__title__ = 'dash.contrib.plugins.image.defaults'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2017 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'DEFAULT_FIT_METHOD',
    'FIT_METHOD_CROP_CENTER',
    'FIT_METHOD_CROP_SCALE',
    'FIT_METHOD_CROP_SMART',
    'FIT_METHOD_FIT_HEIGHT',
    'FIT_METHOD_FIT_WIDTH',
    'FIT_METHODS_CHOICES',
    'FIT_METHODS_CHOICES_WITH_EMPTY_OPTION',
    'IMAGES_UPLOAD_DIR',
)

FIT_METHOD_CROP_SMART = 'smart'
FIT_METHOD_CROP_CENTER = 'center'
FIT_METHOD_CROP_SCALE = 'scale'
FIT_METHOD_FIT_WIDTH = 'fit_width'
FIT_METHOD_FIT_HEIGHT = 'fit_height'

DEFAULT_FIT_METHOD = FIT_METHOD_CROP_CENTER

FIT_METHODS_CHOICES = (
    (FIT_METHOD_CROP_SMART, _("Smart crop")),
    (FIT_METHOD_CROP_CENTER, _("Crop center")),
    (FIT_METHOD_CROP_SCALE, _("Crop scale")),
    (FIT_METHOD_FIT_WIDTH, _("Fit width")),
    (FIT_METHOD_FIT_HEIGHT, _("Fit height")),
)

FIT_METHODS_CHOICES_WITH_EMPTY_OPTION = \
    [('', '---------')] + list(FIT_METHODS_CHOICES)

IMAGES_UPLOAD_DIR = 'dash-image-plugin-images'



from .conf import get_setting

__title__ = 'dash.contrib.plugins.url.settings'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2017 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'BOOKMARK_IMAGE_CHOICES',
    'BOOKMARK_IMAGE_CHOICES_WITH_EMPTY_OPTION'
    'IMAGE_CHOICES',
    'IMAGE_CHOICES_WITH_EMPTY_OPTION',
)

IMAGE_CHOICES = get_setting('IMAGE_CHOICES')
IMAGE_CHOICES_WITH_EMPTY_OPTION = get_setting(
    'IMAGE_CHOICES_WITH_EMPTY_OPTION'
)
BOOKMARK_IMAGE_CHOICES = get_setting('BOOKMARK_IMAGE_CHOICES')
BOOKMARK_IMAGE_CHOICES_WITH_EMPTY_OPTION = get_setting(
    'BOOKMARK_IMAGE_CHOICES_WITH_EMPTY_OPTION'
)



import logging

from django import forms
from django.utils.translation import ugettext_lazy as _

from pif import get_public_ip

from ....base import DashboardPluginFormBase
from ....widgets import BooleanRadioSelect

from .defaults import DEFAULT_CACHE_FOR, DEFAULT_SHOW_TITLE

__title__ = 'dash.contrib.plugins.weather.forms'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2017 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('WeatherForm',)

logger = logging.getLogger(__name__)


class WeatherForm(forms.Form, DashboardPluginFormBase):
    """Form for main ``WeatherPlugin``."""

    plugin_data_fields = [
        ("custom_title", ""),
        ("show_title", DEFAULT_SHOW_TITLE),
        ("cache_for", DEFAULT_CACHE_FOR),
        ("public_ip", ""),
        ("weather_data_json", "")

    ]
    custom_title = forms.CharField(label=_("Custom title"), required=False)
    show_feed_title = forms.BooleanField(label=_("Show title?"),
                                         required=False,
                                         initial=DEFAULT_SHOW_TITLE,
                                         widget=BooleanRadioSelect)
    cache_for = forms.IntegerField(label=_("Cache for"),
                                   required=True,
                                   initial=DEFAULT_CACHE_FOR)
    public_ip = forms.CharField(label=_("Public IP"),
                                required=False,
                                widget=forms.widgets.HiddenInput)
    weather_data_json = forms.CharField(label=_("Weather data JSON"),
                                        required=False,
                                        widget=forms.widgets.HiddenInput)

    def __init__(self, *args, **kwargs):
        super(WeatherForm, self).__init__(*args, **kwargs)

    def save_plugin_data(self, request=None):
        """Save plugin data.

        For showing the weather, we need an IP address. Although we don't
        make it possible for the user to specify it manually, we silently
        obtain it and save into the plugin data.
        """
        if not self.cleaned_data.get('public_ip', None):
            try:
                self.cleaned_data['public_ip'] = get_public_ip()
            except Exception as err:
                if DEBUG:
                    logger.debug(err)



from django.utils.translation import ugettext

__title__ = 'dash.defaults'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2017 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'ACTIVE_LAYOUT',
    'DEBUG',
    'DEFAULT_PLACEHOLDER_EDIT_TEMPLATE_NAME',
    'DEFAULT_PLACEHOLDER_VIEW_TEMPLATE_NAME',
    'DEFAULT_WORKSPACE_NAME',
    'DISPLAY_AUTH_LINK',
    'LAYOUT_CELL_UNITS',
    'LAYOUTS_MODULE_NAME',
    'PLUGIN_CLIPBOARD_KEY',
    'PLUGINS_MODULE_NAME',
    'RAISE_EXCEPTION_WHEN_PERMISSIONS_INSUFFICIENT',
    'RESTRICT_PLUGIN_ACCESS',
    'WAIT_AT_TEST_END',
    'WAIT_BETWEEN_TEST_STEPS',
)


def gettext(val):
    """Fake gettext."""
    return val


# If set to True, plugins would be only accessible by the white-listed user(s)
# or group(s). If set to False, all users have the same access rights to all
# plugins.
RESTRICT_PLUGIN_ACCESS = True

# If set to True, exceptions are raised when user has insufficient permissions.
RAISE_EXCEPTION_WHEN_PERMISSIONS_INSUFFICIENT = True

# Name of the module in which the dash plugins are registered.
PLUGINS_MODULE_NAME = 'dash_plugins'

# Name of the module in which the dash layouts are registered.
LAYOUTS_MODULE_NAME = 'dash_layouts'

# UID of the active layout.
ACTIVE_LAYOUT = 'android'

# Allowed layout cell units.
LAYOUT_CELL_UNITS = (
    'em',
    'px',
    'pt',
    '%',
    'rem',
    'in',
    'cm',
    'mm',
    'ex',
    'pc'
)

# Name of the default dashboard workspace (no workspace).
DEFAULT_WORKSPACE_NAME = 'Default'

DEFAULT_PLACEHOLDER_VIEW_TEMPLATE_NAME = \
    'dash/layouts/base_placeholder_view.html'
DEFAULT_PLACEHOLDER_EDIT_TEMPLATE_NAME = \
    'dash/layouts/base_placeholder_edit.html'

# If set to True, the logout link is shown in the menu.
DISPLAY_AUTH_LINK = True

WAIT_BETWEEN_TEST_STEPS = 2
WAIT_AT_TEST_END = 4

# Key of the clipboard variable used in sessions to store the plugin clipboard
# data.
PLUGIN_CLIPBOARD_KEY = 'dash_plugin_clipboard'

DEBUG = False



# from optparse import OptionParser
from time import sleep
import unittest

from django.conf import settings
from django.contrib.auth import get_user_model
# from django.contrib.staticfiles.management.commands import collectstatic
from django.core.management import call_command
from django.test import (
    TestCase,
    RequestFactory,
    LiveServerTestCase,
    # Client,
)

from selenium import webdriver
# from selenium.common.exceptions import WebDriverException
from selenium.webdriver.firefox.firefox_binary import FirefoxBinary
# from selenium.webdriver.firefox.webdriver import WebDriver
from selenium.webdriver.support.wait import WebDriverWait

from six import print_

from .base import (
    get_layout,
    get_registered_layouts,
    # get_registered_plugins,
    # layout_registry,
    # plugin_registry,
)
# from .discover import autodiscover
# from .management.commands import dash_sync_plugins
from .models import DashboardEntry
from .settings import WAIT_BETWEEN_TEST_STEPS, WAIT_AT_TEST_END
from .utils import get_occupied_cells, get_user_plugins

__title__ = 'dash.tests'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2017 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'


DASH_TEST_USER_USERNAME = 'test_admin'
DASH_TEST_USER_PASSWORD = 'test'
PRINT_INFO = True
TRACK_TIME = False


def print_info(func):
    """Prints some useful info."""
    if not PRINT_INFO:
        return func

    def inner(self, *args, **kwargs):
        result = func(self, *args, **kwargs)

        print_('\n{0}'.format(func.__name__))
        print_('============================')
        if func.__doc__:
            print_('""" {0} """'.format(func.__doc__.strip()))
        print_('----------------------------')
        if result is not None:
            print_(result)
        print_('\n')

        return result
    return inner


def create_dashboard_user():
    """
    Create a user for testing the dashboard.

    TODO: At the moment an admin account is being tested. Automated tests with
    diverse accounts are to be implemented.
    """
    User = get_user_model()
    u = User()
    u.username = DASH_TEST_USER_USERNAME
    u.email = 'admin@dev.django-dash.com'
    u.is_superuser = True
    u.is_staff = True
    u.set_password(DASH_TEST_USER_PASSWORD)

    try:
        u.save()
    except Exception as err:
        pass


DASH_SET_UP = False


def setup_dash():
    """Set up dash."""
    call_command('collectstatic', verbosity=3, interactive=False)
    call_command('dash_sync_plugins', verbosity=3, interactive=False)


class DashCoreTest(TestCase):
    """Tests of django-dash core functionality."""

    def setUp(self):
        setup_dash()

    @print_info
    def test_01_registered_layouts(self):
        """Test registered layouts (`get_registered_layouts`)."""
        res = get_registered_layouts()
        self.assertTrue(len(res) > 0)
        return res

    @print_info
    def test_02_active_layout(self):
        """Test active layout (`get_layout`)."""
        layout_cls = get_layout()
        self.assertTrue(layout_cls is not None)
        return layout_cls

    @print_info
    def test_03_get_layout_placeholders(self):
        """Test active layout placeholders (`get_placeholder_instances`)."""
        layout_cls = get_layout()
        layout = layout_cls()
        res = layout.get_placeholder_instances()
        self.assertTrue(len(res) > 0)
        return res

    @print_info
    def test_04_active_layout_render_for_view(self):
        """Test active layout render (`render_for_view`)."""
        try:
            # Create dashboard user
            create_dashboard_user()
        except Exception:
            pass

        User = get_user_model()
        # Getting the admin (user with plugin data)
        user = User.objects.get(username=DASH_TEST_USER_USERNAME)

        # Faking the Django request
        request_factory = RequestFactory()
        request = request_factory.get('/dashboard/', data={'user': user})
        request.user = user
        workspace = None

        # Getting the list of plugins that user is allowed to use.
        registered_plugins = get_user_plugins(request.user)
        user_plugin_uids = [uid for uid, repr in registered_plugins]

        layout = get_layout(as_instance=True)

        # Fetching all dashboard entries for user and freezeing the queryset
        dashboard_entries = DashboardEntry \
            ._default_manager \
            .get_for_user(user=request.user,
                          layout_uid=layout.uid,
                          workspace=workspace) \
            .select_related('workspace', 'user') \
            .filter(plugin_uid__in=user_plugin_uids) \
            .order_by('placeholder_uid', 'position')[:]

        res = layout.render_for_view(dashboard_entries=dashboard_entries,
                                     request=request)
        return res

    @print_info
    def test_05_get_occupied_cells(self):
        """Test ``dash.utils.get_occupied_cells``."""
        # Fake dashboard entry
        class Entry(object):
            pass

        layout = get_layout(as_instance=True)
        placeholder = layout.get_placeholder('main')

        res = []

        if 'android' == layout.uid:
            # *********** First test
            # 2 x 2 widget
            r = get_occupied_cells(layout, placeholder, 'memo_2x2', 3)

            self.assertEqual(r, [3, 4, 9, 10])

            res.append(r)

            # *********** Second test

            # 3 x 3 widget
            r = get_occupied_cells(layout, placeholder, 'memo_3x3', 16)

            self.assertEqual(r, [16, 17, 18, 22, 23, 24, 28, 29, 30])

            res.append(r)

            # *********** Third test (the nasty one)

            # 3 x 3 widget
            # r = get_occupied_cells(layout, placeholder, 'memo_3x3', 17)

            # self.assertEqual(r, [16, 17, 18, 22, 23, 24, 28, 29, 30])

            # res.append(r)

        return res


class DashBrowserTest(LiveServerTestCase):
    """django-dash browser tests.

    TODO: At the moment is done for admin only. Normal users shall be tested
    as well for plugin security workflow (permissions system used).
    """
    try:
        LIVE_SERVER_URL = settings.LIVE_SERVER_URL
    except Exception as e:
        LIVE_SERVER_URL = None

    @classmethod
    def setUpClass(cls):
        """Set up class."""
        # cls.selenium = WebDriver()
        firefox_bin_path = getattr(settings, 'FIREFOX_BIN_PATH', None)
        phantom_js_executable_path = getattr(
            settings, 'PHANTOM_JS_EXECUTABLE_PATH', None
        )
        if phantom_js_executable_path is not None:
            if phantom_js_executable_path:
                cls.selenium = webdriver.PhantomJS(
                    executable_path=phantom_js_executable_path
                )
            else:
                cls.selenium = webdriver.PhantomJS()
        elif firefox_bin_path:
            binary = FirefoxBinary(firefox_bin_path)
            cls.selenium = webdriver.Firefox(firefox_binary=binary)
        else:
            cls.selenium = webdriver.Firefox()

        setup_dash()

        super(DashBrowserTest, cls).setUpClass()

    @classmethod
    def tearDownClass(cls):
        """Tear down class."""
        try:
            cls.selenium.quit()
        except Exception as err:
            print(err)

        super(DashBrowserTest, cls).tearDownClass()
        call_command('flush', verbosity=0, interactive=False,
                     reset_sequences=False,
                     allow_cascade=False,
                     inhibit_post_migrate=False)

    def _click(self, element):
        """Click on any element."""
        self.selenium.execute_script("$(arguments[0]).click();", element)

    def _agressive_click(self, element):
        """Agressive click."""
        link = element.get_attribute('href')
        self.selenium.get(link)

    def __add_plugin_widget_test(self,
                                 position,
                                 plugin_widget_name,
                                 plugin_widget_name_with_dimensions,
                                 plugin_widget_css_class,
                                 added_plugin_widget_css_classes,
                                 form_data={},
                                 form_hook_func=None):
        """
        Test add any single plugin.

        :param string position: Example value "col-1 row-1"
        :param string plugin_widget_name: Example value "Dummy".
        :param string plugin_widget_name_with_dimensions: Example
            value "Dummy (1x1)".
        :param string plugin_widget_css_class: Example value "plugin-dummy1x1".
        :param list added_plugin_widget_css_classes: Example
            value ['width-1', 'height-1'].
        :param dict form_data: Example value
            {
                'title': "Lorem",
                'text': "Lorem ipsum dolor sit amet"
            }.
        :param callable form_hook_func: Function to when add form is opened (
            to populate the data).

        :example:

        Test 1x1 URL plugin widget::

            def choose_url_image():
                # Hook function to select an image for test 1x1
                # URL plugin widget.
                image_input = self.selenium.find_element_by_xpath(
                    '//select[@name="image"]/option[@value="icon-coffee"]'
                )
                self.assertTrue(image_input is not None)
                image_input.click()


            self.__add_plugin_widget_test(
                position="col-1 row-1",
                plugin_widget_name="URL",
                plugin_widget_name_with_dimensions="URL (1x1)",
                plugin_widget_css_class="plugin-url_1x1",
                added_plugin_widget_css_classes ('width-1', 'height-1'),
                form_data={
                    'title': "Test 1x1 URL",
                    'url': "http://delusionalinsanity.com/portfolio/"
                },
                form_hook_func=choose_url_image
            )

        Test 2x1 Dummy plugin widget::

            self.__add_plugin_widget_test(
                position="col-2 row-1",
                plugin_widget_name="Dummy",
                plugin_widget_name_with_dimensions="Dummy (2x1)",
                plugin_widget_css_class="plugin-dummy2x1",
                added_plugin_widget_css_classes=('width-2', 'height-1')
            )

        Test 3x3 Memo plugin widget::

            self.__add_plugin_widget_test(
                position="col-4 row-1",
                plugin_widget_name="Memo",
                plugin_widget_name_with_dimensions="Memo (3x3)",
                plugin_widget_css_class="plugin-memo_3x3",
                added_plugin_widget_css_classes=('width-3', 'height-3'),
                form_data={
                    'title': "Lorem",
                    'text': "Lorem ipsum dolor sit amet"
                }
            )

        Test 3x3 Video plugin widget::

            self.__add_plugin_widget_test(
                position="col-1 row-2",
                plugin_widget_name="Video",
                plugin_widget_name_with_dimensions="Video (3x3)",
                plugin_widget_css_class="plugin-video_3x3",
                added_plugin_widget_css_classes=('width-3', 'height-3'),
                form_data = {
                    'title': "Test 3x3 video",
                    'url': "http://www.youtube.com/watch?v=8GVIui0JK0M"
                }
            )
        """
        # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        # +++++++++++++ Step 1: Dashboard user logs in ++++++++++++++++++
        # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        live_server_url = self.LIVE_SERVER_URL \
            if self.LIVE_SERVER_URL \
            else self.live_server_url
        self.selenium.get('{0}{1}'.format(live_server_url, settings.LOGIN_URL))
        self.selenium.maximize_window()
        username_input = self.selenium.find_element_by_name("username")
        username_input.send_keys(DASH_TEST_USER_USERNAME)
        password_input = self.selenium.find_element_by_name("password")
        password_input.send_keys(DASH_TEST_USER_PASSWORD)
        submit_button = self.selenium.find_element_by_xpath(
            '//button[@type="submit"]'
        )
        submit_button.click()

        # Wait until the list view opens
        WebDriverWait(self.selenium, timeout=4).until(
            lambda driver: driver.find_element_by_xpath(
                '//body[contains(@class, "layout")]'
            )
        )

        # Click the button to go to dashboard edit
        dashboard_edit_link = self.selenium.find_element_by_xpath(
            '//a[contains(@class, "menu-dashboard-edit")]'
        )
        dashboard_edit_link.click()

        # Wait until the dashboard edit view opens
        WebDriverWait(self.selenium, timeout=4).until(
            lambda driver: driver.find_element_by_xpath(
                '//body[contains(@class, "layout")]'
            )
        )

        # Click the add widget button to add a new widget to the dashboard
        add_plugin_widget_div = self.selenium.find_element_by_xpath(
            '//div[contains(@class, "{0}")]'.format(position)
        )
        add_plugin_link = add_plugin_widget_div.find_element_by_class_name(
            'add-plugin'
        )
        # add_plugin_link.click()
        self._click(add_plugin_link)

        # Wait until the add widget view opens
        WebDriverWait(self.selenium, timeout=8).until(
            lambda driver: driver.find_element_by_xpath(
                '//a[text()="{0}"]'.format(plugin_widget_name_with_dimensions)
            )
        )

        # Wait until the accordion is really loaded
        WebDriverWait(self.selenium, timeout=4).until(
            lambda driver: driver.find_element_by_id('accordion')
        )

        # Add a dummy (1x1) widget
        add_dummy_plugin_widget = self.selenium.find_element_by_xpath(
            '//a[text()="{0}"]'.format(plugin_widget_name_with_dimensions)
        )

        self.selenium.get('{0}'.format(
            add_dummy_plugin_widget.get_attribute('href'))
        )

        # Wait until the add dummy widget form opens
        WebDriverWait(self.selenium, timeout=4).until(
            lambda driver: driver.find_element_by_xpath(
                '//body[contains(@class, "standalone")]'
            )
        )

        # Filling with test data
        if form_data:
            for field_name, field_value in form_data.items():
                field_input = self.selenium.find_element_by_name(field_name)
                field_input.send_keys(field_value)

        # If form_hook_func (callable) is specified, call it.
        if form_hook_func:
            form_hook_func()

        # Click add widget button
        add_widget_button = self.selenium.find_element_by_xpath(
            '//button[@type="submit"]'
        )
        # add_widget_button.click()
        self._click(add_widget_button)  # Unsure

        # Wait until the edit dashboard page opens
        WebDriverWait(self.selenium, timeout=8).until(
            lambda driver: driver.find_element_by_xpath(
                '//div[contains(@class, "{0}")]'.format(
                    plugin_widget_css_class
                )
            )
        )

        # Make sure the success message is there
        self.selenium.find_element_by_xpath(
            """//li[text()='The dashboard widget "{0}" was added """
            """successfully.']""".format(plugin_widget_name)
        )

        dummy_plugin_widget = self.selenium.find_element_by_xpath(
            '//div[contains(@class, "{0}")]'.format(plugin_widget_css_class)
        )
        dummy_plugin_widget_classes = dummy_plugin_widget.get_attribute(
            'class'
        )
        dummy_plugin_widget_classes = dummy_plugin_widget_classes.split(' ')
        for added_plugin_widget_css_class in added_plugin_widget_css_classes:
            self.assertTrue(added_plugin_widget_css_class
                            in dummy_plugin_widget_classes)

    def __add_dashboard_entry_test(self, wait=0):
        """
        Add dashboard entry test.

        :param int wait: Number of seconds to sleep at the end of the test.
        """
        flow = []

        try:
            # Create dashboard user
            create_dashboard_user()
        except Exception:
            pass

        # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        # +++++++++++++ Step 1: Dashboard user logs in ++++++++++++++++++
        # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

        def choose_url_image():
            """Hook function to select an image.

            For test 1x1 URL plugin widget.
            """
            image_input = self.selenium.find_element_by_xpath(
                '//select[@name="image"]/option[@value="icon-coffee"]'
            )
            self.assertTrue(image_input is not None)
            # image_input.click()
            self._click(image_input)  # Unsure

        # Test 1x1 URL plugin widget
        self.__add_plugin_widget_test(
            position="col-1 row-1",
            plugin_widget_name="URL",
            plugin_widget_name_with_dimensions="URL (1x1)",
            plugin_widget_css_class="plugin-url_1x1",
            added_plugin_widget_css_classes=('width-1', 'height-1'),
            form_data={'title': "Test 1x1 URL",
                       'url': "http://delusionalinsanity.com/portfolio/"},
            form_hook_func=choose_url_image
        )

        # Test 2x1 Dummy plugin widget
        self.__add_plugin_widget_test(
            position="col-2 row-1",
            plugin_widget_name="Dummy",
            plugin_widget_name_with_dimensions="Dummy (2x1)",
            plugin_widget_css_class="plugin-dummy_2x1",
            added_plugin_widget_css_classes=('width-2', 'height-1')
        )

        # Test 3x3 Memo plugin widget
        self.__add_plugin_widget_test(
            position="col-4 row-1",
            plugin_widget_name="Memo",
            plugin_widget_name_with_dimensions="Memo (3x3)",
            plugin_widget_css_class="plugin-memo_3x3",
            added_plugin_widget_css_classes=('width-3', 'height-3'),
            form_data={'title': "Test 3x3 memo",
                       'text': "Lorem ipsum dolor sit amet."}
        )

        # Test 3x3 Video plugin widget
        self.__add_plugin_widget_test(
            position="col-1 row-2",
            plugin_widget_name="Video",
            plugin_widget_name_with_dimensions="Video (3x3)",
            plugin_widget_css_class="plugin-video_3x3",
            added_plugin_widget_css_classes=('width-3', 'height-3'),
            form_data={'title': "Test 3x3 video",
                       'url': "http://www.youtube.com/watch?v=8GVIui0JK0M"}
        )

        if wait:
            sleep(wait)

        return flow

    def __edit_plugin_widget_test(self,
                                  plugin_widget_name,
                                  plugin_widget_css_class,
                                  form_data={},
                                  form_hook_func=None):
        """Test edit any single plugin.

        :param string plugin_widget_name: Example value "Dummy".
        :param string plugin_widget_css_class: Example value "plugin-dummy1x1".
        :param dict form_data: Example
            value {'title': "Lorem", 'text': "Lorem ipsum dolor sit amet"}.
        :param callable form_hook_func: Function to when edit form is
            opened (to populate the data).

        :example:

        Test 1x1 URL plugin widget::

            def choose_url_image():
                # Hook function to select an image for test 1x1 URL plugin
                # widget.
                image_input = self.selenium.find_element_by_xpath(
                    '//select[@name="image"]/option[@value="icon-coffee"]'
                )
                self.assertTrue(image_input is not None)
                image_input.click()

            self.__edit_plugin_widget_test(
                plugin_widget_name="URL",
                plugin_widget_css_class="plugin-url_1x1",
                form_data = {
                    'title': "Test 1x1 URL",
                    'url': "http://delusionalinsanity.com/portfolio/"
                },
                form_hook_func=choose_url_image
            )

        Test 2x1 Dummy plugin widget::

            self.__edit_plugin_widget_test(
                plugin_widget_name="Dummy",
                plugin_widget_css_class="plugin-dummy_2x1"
            )

        Test 3x3 Memo plugin widget::

            self.__edit_plugin_widget_test(
                plugin_widget_name="Memo",
                plugin_widget_css_class="plugin-memo_3x3",
                form_data={'title': "Lorem",
                           'text': "Lorem ipsum dolor sit amet"}
            )

        Test 3x3 Video plugin widget::

            self.__edit_plugin_widget_test(
                plugin_widget_name="Video",
                plugin_widget_css_class="plugin-video_3x3",
                form_data={
                    'title': "Test 3x3 video",
                    'url': "http://www.youtube.com/watch?v=8GVIui0JK0M"
                }
            )
        """
        # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        # +++++++++++++ Step 2: User edits the plugin widgets +++++++++++
        # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

        # Click the add widget button to edit the widget on the dashboard
        edit_plugin_widget_div = self.selenium.find_element_by_xpath(
            '//div[contains(@class, "{0}")]'.format(plugin_widget_css_class)
        )
        edit_plugin_link = edit_plugin_widget_div.find_element_by_class_name(
            'edit-plugin'
        )
        # edit_plugin_link.click()
        # self._click(edit_plugin_link)  # Unsure
        self._agressive_click(edit_plugin_link)  # Unsure

        # Wait until the edit widget form opens
        WebDriverWait(self.selenium, timeout=8).until(
            lambda driver: driver.find_element_by_xpath(
                '//body[contains(@class, "standalone")]'
            )
        )

        # Filling with test data
        if form_data:
            for field_name, field_value in form_data.items():
                field_input = self.selenium.find_element_by_name(field_name)
                field_input.clear()
                field_input.send_keys(field_value)

        # If form_hook_func (callable) is specified, call it.
        if form_hook_func:
            form_hook_func()

        # Click add widget button
        widget_button = self.selenium.find_element_by_xpath(
            '//button[@type="submit"]'
        )
        # widget_button.click()
        self._click(widget_button)  # Unsure

        # Wait until the edit dashboard page opens
        WebDriverWait(self.selenium, timeout=4).until(
            lambda driver: driver.find_element_by_xpath(
                '//div[contains(@class, "{0}")]'.format(
                    plugin_widget_css_class
                )
            )
        )

        # Make sure the success message is there
        self.selenium.find_element_by_xpath(
            """//li[text()='The dashboard widget "{0}" was edited """
            """successfully.']""".format(plugin_widget_name)
        )

        dummy_plugin_widget = self.selenium.find_element_by_xpath(
            '//div[contains(@class, "{0}")]'.format(plugin_widget_css_class)
        )

    def __edit_dashboard_entry_test(self, wait=0):
        """Edit dashboard entry test.

        :param int wait: Number of seconds to sleep at the end of the test.
        """
        flow = []

        # Test 1x1 URL plugin widget::

        def choose_url_image():
            # Hook function to select an image for test 1x1 URL plugin widget.
            image_input = self.selenium.find_element_by_xpath(
                '//select[@name="image"]/option[@value="icon-camera"]'
            )
            self.assertTrue(image_input is not None)
            # image_input.click()
            self._click(image_input)  # Unsure

        self.__edit_plugin_widget_test(
            plugin_widget_name="URL",
            plugin_widget_css_class="plugin-url_1x1",
            form_data={
                'title': "Edited test 1x1 URL",
                'url': "http://foreverchild.info/"
            },
            form_hook_func=choose_url_image
        )

        # Test 2x1 Dummy plugin widget::

        self.__edit_plugin_widget_test(
            plugin_widget_name="Dummy",
            plugin_widget_css_class="plugin-dummy_2x1"
        )

        # Test 3x3 Memo plugin widget::

        self.__edit_plugin_widget_test(
            plugin_widget_name="Memo",
            plugin_widget_css_class="plugin-memo_3x3",
            form_data={
                'title': "Edited lorem",
                'text': "Edited lorem ipsum dolor sit amet"
            }
        )

        # Test 3x3 Video plugin widget::

        self.__edit_plugin_widget_test(
            plugin_widget_name="Video",
            plugin_widget_css_class="plugin-video_3x3",
            form_data={
                'title': "Edited test 3x3 video",
                'url': "http://www.youtube.com/watch?v=veOhHqVWwP4"
            }
        )

        if wait:
            sleep(wait)

        return flow

    def __delete_plugin_widget_test(self, plugin_widget_css_class):
        """Test delete any single plugin.

        :param string plugin_widget_css_class: Example value "plugin-dummy1x1".

        :example:

        Test 1x1 URL plugin widget::

            self.__delete_plugin_widget_test(
                plugin_widget_css_class="plugin-url_1x1"
            )

        Test 2x1 Dummy plugin widget::

            self.__delete_plugin_widget_test(
                plugin_widget_css_class="plugin-dummy_2x1"
            )

        Test 3x3 Memo plugin widget::

            self.__delete_plugin_widget_test(
                plugin_widget_css_class="plugin-memo_3x3"
            )

        Test 3x3 Video plugin widget::

            self.__delete_plugin_widget_test(
                plugin_widget_css_class="plugin-video_3x3"
            )
        """
        # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        # +++++++++++++ Step 2: User deletes the plugin widgets +++++++++
        # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

        # Click the add widget button to edit the widget on the dashboard
        edit_plugin_widget_div = self.selenium.find_element_by_xpath(
            '//div[contains(@class, "{0}")]'.format(plugin_widget_css_class)
        )

        remove_plugin_link = edit_plugin_widget_div.find_element_by_class_name(
            'remove-plugin'
        )
        # remove_plugin_link.click()
        self._click(remove_plugin_link)

        # Wait until the edit dashboard page opens
        WebDriverWait(self.selenium, timeout=8).until(
            lambda driver: driver.find_element_by_xpath(
                '//body[contains(@class, "layout")]'
            )
        )

        found = False
        try:
            dummy_plugin_widget = self.selenium.find_element_by_xpath(
                '//div[contains(@class, "{0}")]'.format(
                    plugin_widget_css_class
                )
            )
            found = True
        except Exception:
            pass

        self.assertTrue(not found)

    def __delete_dashboard_entry_test(self, wait=0):
        """Delete dashboard entry test.

        :param int wait: Number of seconds to sleep at the end of the test.
        """
        flow = []

        # Test 1x1 URL plugin widget::

        self.__delete_plugin_widget_test(
            plugin_widget_css_class="plugin-url_1x1"
        )

        # Test 2x1 Dummy plugin widget::

        self.__delete_plugin_widget_test(
            plugin_widget_css_class="plugin-dummy_2x1"
        )

        # Test 3x3 Memo plugin widget::

        self.__delete_plugin_widget_test(
            plugin_widget_css_class="plugin-memo_3x3"
        )

        # Test 3x3 Video plugin widget::

        self.__delete_plugin_widget_test(
            plugin_widget_css_class="plugin-video_3x3"
        )

        if wait:
            sleep(wait)

        return flow

    @print_info
    def test_01_add_dashboard_entry(self):
        """Add dashboard entry test."""
        return self.__add_dashboard_entry_test(wait=WAIT_AT_TEST_END)

    @print_info
    def test_02_edit_dashboard_entry(self):
        """Edit dashboard entry test."""
        self.__add_dashboard_entry_test(wait=WAIT_BETWEEN_TEST_STEPS)
        return self.__edit_dashboard_entry_test(wait=WAIT_AT_TEST_END)

    @print_info
    def test_03_delete_dashboard_entry(self):
        """Delete dashboard entry test."""
        self.__add_dashboard_entry_test(wait=WAIT_BETWEEN_TEST_STEPS)
        return self.__delete_dashboard_entry_test(wait=WAIT_AT_TEST_END)


if __name__ == '__main__':
    unittest.main()



from django.contrib import messages
from django.contrib.auth.decorators import login_required, permission_required
from django.core.exceptions import ObjectDoesNotExist
from django.http import Http404, HttpResponse
from django.shortcuts import redirect
from django.template import RequestContext
from django.utils.translation import ugettext, ugettext_lazy as _

from nine import versions

from .base import (
    get_layout,
    plugin_registry,
    # plugin_widget_registry,
    # PluginWidgetRegistry,
    validate_placeholder_uid,
    validate_plugin_uid,
)
from .clipboard import (
    can_paste_entry_from_clipboard,
    copy_entry_to_clipboard,
    cut_entry_to_clipboard,
    get_plugin_data_from_clipboard,
    paste_entry_from_clipboard,
)
from .decorators import (
    edit_dashboard_permission_required,
    permissions_required,
    SATISFY_ALL,
    use_clipboard_permission_required,
)
from .forms import DashboardWorkspaceForm, DashboardSettingsForm
from .helpers import (
    clean_plugin_data,
    iterable_to_dict,
    lists_overlap,
    safe_text,
    slugify_workspace,
)
from .models import DashboardEntry, DashboardWorkspace
from .settings import RAISE_EXCEPTION_WHEN_PERMISSIONS_INSUFFICIENT
from .utils import (
    build_cells_matrix,
    clone_workspace,
    get_occupied_cells,
    get_or_create_dashboard_settings,
    get_public_dashboard_url,
    get_user_plugins,
    get_widgets,
    get_workspaces,
)
from .json_package import json

if versions.DJANGO_GTE_1_10:
    from django.shortcuts import render
    from django.urls import reverse
else:
    from django.core.urlresolvers import reverse
    from django.shortcuts import render_to_response

__title__ = 'dash.views'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2017 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'add_dashboard_entry',
    'clone_dashboard_workspace',
    'copy_dashboard_entry',
    'create_dashboard_workspace',
    'cut_dashboard_entry',
    'dashboard',
    'dashboard_workspaces',
    'delete_dashboard_entry',
    'delete_dashboard_workspace',
    'edit_dashboard',
    'edit_dashboard_entry',
    'edit_dashboard_settings',
    'edit_dashboard_workspace',
    'paste_dashboard_entry',
    'plugin_widgets',
)

# ***************************************************************************
# ***************************************************************************
# **************************** Dashboard views ******************************
# ***************************************************************************
# ***************************************************************************


@login_required
def dashboard(request, workspace=None):
    """Dashboard.

    :param django.http.HttpRequest request:
    :param string workspace: Workspace slug. If given, the workspace loaded.
        Otherwise we deal with no workspace.
    :return django.http.HttpResponse:
    """
    # Getting the list of plugins that user is allowed to use.
    registered_plugins = get_user_plugins(request.user)
    user_plugin_uids = [uid for uid, repr in registered_plugins]

    # Getting dashboard settings for the user. Then get users' layout.
    dashboard_settings = get_or_create_dashboard_settings(request.user)

    workspaces = get_workspaces(
        request.user,
        dashboard_settings.layout_uid,
        workspace,
        different_layouts=dashboard_settings.allow_different_layouts
    )

    layout = get_layout(
        layout_uid=(
            workspaces['current_workspace'].layout_uid
            if workspaces['current_workspace']
            else dashboard_settings.layout_uid
        ),
        as_instance=True
    )

    # If workspace with slug given is not found in the list of workspaces
    # redirect to the default dashboard.
    if workspaces['current_workspace_not_found']:
        msg = _('The workspace with slug "{0}" does '
                'not belong to layout "{1}".').format(workspace, layout.name)
        if dashboard_settings.allow_different_layouts:
            msg = _('The workspace with slug "{0}" does not exist').format(
                workspace
            )
        messages.info(request, msg)
        return redirect('dash.edit_dashboard')

    # Getting the (frozen) queryset.
    dashboard_entries = DashboardEntry._default_manager \
        .get_for_user(user=request.user,
                      layout_uid=layout.uid,
                      workspace=workspace) \
        .select_related('workspace', 'user') \
        .filter(plugin_uid__in=user_plugin_uids) \
        .order_by('placeholder_uid', 'position')[:]

    placeholders = layout.get_placeholder_instances(
        dashboard_entries, request=request
    )

    layout.collect_widget_media(dashboard_entries)

    context = {
        'placeholders': placeholders,
        'placeholders_dict': iterable_to_dict(placeholders,
                                              key_attr_name='uid'),
        'css': layout.get_css(placeholders),
        'layout': layout,
        'dashboard_settings': dashboard_settings
    }

    context.update(workspaces)

    context.update(
        {'public_dashboard_url': get_public_dashboard_url(dashboard_settings)}
    )

    template_name = layout.get_view_template_name(request)

    if versions.DJANGO_GTE_1_10:
        return render(request, template_name, context)
    else:
        return render_to_response(
            template_name, context, context_instance=RequestContext(request)
        )


@login_required
@edit_dashboard_permission_required()
def edit_dashboard(request, workspace=None):
    """Edit dashboard.

    :param django.http.HttpRequest request:
    :param string workspace: Workspace slug. If given, the workspace loaded.
        Otherwise we deal with no workspace.
    :return django.http.HttpResponse:
    """
    # Getting the list of plugins that user is allowed to use.
    registered_plugins = get_user_plugins(request.user)
    user_plugin_uids = [uid for uid, repr in registered_plugins]

    # Getting dashboard settings for the user. Then get users' layout.
    dashboard_settings = get_or_create_dashboard_settings(request.user)

    workspaces = get_workspaces(
        request.user,
        dashboard_settings.layout_uid,
        workspace,
        different_layouts=dashboard_settings.allow_different_layouts,
    )

    layout = get_layout(
        layout_uid=(
            workspaces['current_workspace'].layout_uid
            if workspaces['current_workspace']
            else dashboard_settings.layout_uid
        ),
        as_instance=True
    )

    # If workspace with slug given is not found in the list of workspaces
    # redirect to the default dashboard.
    if workspaces['current_workspace_not_found']:
        messages.info(
            request,
            _('The workspace with slug "{0}" does '
              'not belong to layout "{1}".').format(workspace, layout.name)
            )
        return redirect('dash.edit_dashboard')

    # Getting the (frozen) queryset.
    dashboard_entries = DashboardEntry._default_manager \
        .get_for_user(user=request.user,
                      layout_uid=layout.uid,
                      workspace=workspace) \
        .select_related('workspace', 'user') \
        .filter(plugin_uid__in=user_plugin_uids) \
        .order_by('placeholder_uid', 'position')[:]

    placeholders = layout.get_placeholder_instances(
        dashboard_entries, workspace=workspace, request=request
    )

    layout.collect_widget_media(dashboard_entries)

    context = {
        'placeholders': placeholders,
        'placeholders_dict': iterable_to_dict(placeholders,
                                              key_attr_name='uid'),
        'css': layout.get_css(placeholders),
        'layout': layout,
        'edit_mode': True,
        'dashboard_settings': dashboard_settings
    }

    context.update(workspaces)

    context.update(
        {'public_dashboard_url': get_public_dashboard_url(dashboard_settings)}
    )

    template_name = layout.get_edit_template_name(request)

    if versions.DJANGO_GTE_1_10:
        return render(request, template_name, context)
    else:
        return render_to_response(
            template_name, context, context_instance=RequestContext(request)
        )


# ***************************************************************************
# ***************************************************************************
# ********************* Add/edit/delete dashboard entry *********************
# ***************************************************************************
# ***************************************************************************


@login_required
@permission_required('dash.add_dashboardentry')
def add_dashboard_entry(request,
                        placeholder_uid,
                        plugin_uid,
                        workspace=None,
                        position=None,
                        template_name='dash/add_dashboard_entry.html',
                        template_name_ajax='dash/add_dashboard_entry_ajax'
                                           '.html'):
    """Add dashboard entry.

    :param django.http.HttpRequest request:
    :param string placeholder_uid: Placeholder UID.
    :param string plugin_uid: Plugin UID.
    :param string workspace: Workspace slug.
    :param int position: If given, provided as position for the
        plugin (conflict resolution should take place).
    :param string template_name:
    :param string template_name_ajax: Template used for AJAX requests.
    :return django.http.HttpResponse:
    """
    # Getting dashboard settings for the user. Then get users' layout.
    dashboard_settings = get_or_create_dashboard_settings(request.user)

    if workspace:
        workspace_slug = slugify_workspace(workspace)
        filters = {
            'slug': workspace_slug,
            'user': request.user,
        }
        if not dashboard_settings.allow_different_layouts:
            filters.update({
                'layout_uid': dashboard_settings.layout_uid,
            })
        try:
            workspace = DashboardWorkspace._default_manager.get(**filters)
        except ObjectDoesNotExist as e:
            if dashboard_settings.allow_different_layouts:
                message = _('The workspace with slug "{0}" was not found.'
                            '').format(workspace_slug)
            else:
                message = _(
                    'The workspace with slug "{0}" does not belong to '
                    'layout "{1}".'
                ).format(workspace_slug, layout.name)
            messages.info(request, message)
            return redirect('dash.edit_dashboard')

    if dashboard_settings.allow_different_layouts and workspace:
        layout_uid = workspace.layout_uid
    else:
        layout_uid = dashboard_settings.layout_uid

    layout = get_layout(layout_uid=layout_uid, as_instance=True)

    if not validate_placeholder_uid(layout, placeholder_uid):
        raise Http404(ugettext("Invalid placeholder: {0}").format(placeholder))

    if not validate_plugin_uid(plugin_uid):
        raise Http404(ugettext("Invalid plugin name: {0}").format(plugin_uid))

    placeholder = layout.get_placeholder(placeholder_uid)

    # Cell that would be occupied by the plugin upon addition.
    widget_occupied_cells = get_occupied_cells(
        layout,
        placeholder,
        plugin_uid,
        position,
        check_boundaries=True,
        fail_silently=True
    )

    # Cells currently occupied in the workspace given.
    occupied_cells = build_cells_matrix(
        request.user,
        layout,
        placeholder,
        workspace
    )

    # Checking if it's still possible to insert a widget.
    if widget_occupied_cells is False \
       or lists_overlap(widget_occupied_cells, occupied_cells):

        raise Http404(ugettext("Collisions detected"))

    plugin = plugin_registry.get(plugin_uid)(layout.uid, placeholder_uid)
    plugin.request = request

    if plugin.add_form_template:
        template_name = plugin.add_form_template

    # Template context
    context = {
        'layout': layout,
        'dashboard_settings': dashboard_settings
    }

    obj = DashboardEntry()
    obj.layout_uid = layout.uid
    obj.placeholder_uid = placeholder_uid
    obj.plugin_uid = plugin_uid
    obj.user = request.user
    obj.workspace = workspace

    # If plugin has form, it is configurable which means we have to load the
    # plugin form and validate user input.
    plugin_form = plugin.get_form()
    if plugin_form:
        # If POST request and form data is valid, save the data and redirect
        # to the dashboard edit.
        if request.method == 'POST':
            form = plugin.get_initialised_create_form_or_404(
                data=request.POST, files=request.FILES
            )
            if form.is_valid():
                # Saving the plugin form data.
                form.save_plugin_data(request=request)

                # Getting the plugin data.
                obj.plugin_data = form.get_plugin_data(request=request)

                # If position given, use it.
                try:
                    position = int(position)
                except Exception:
                    position = None

                if position:
                    obj.position = position

                # Save the object.
                obj.save()

                messages.info(
                    request,
                    _('The dashboard widget "{0}" was added successfully.'
                      '').format(plugin.name)
                )

                # Redirect to the dashboard view.
                if obj.workspace:
                    return redirect(
                        'dash.edit_dashboard', workspace=obj.workspace.slug
                    )
                else:
                    return redirect('dash.edit_dashboard')

        # If POST but data invalid, show the form with errors.
        else:
            form = plugin.get_initialised_create_form_or_404()

        context.update(
            {'form': form, 'plugin_uid': plugin_uid, 'plugin': plugin}
        )

    # If plugin is not configurable, it's just saved as is.
    else:
        obj.save()
        return redirect('dash.edit_dashboard')

    if layout.add_dashboard_entry_ajax_template_name:
        template_name_ajax = layout.add_dashboard_entry_ajax_template_name

    context.update(
        {'add_dashboard_entry_ajax_template_name': template_name_ajax}
    )

    if request.is_ajax():
        template_name = template_name_ajax
    elif layout.add_dashboard_entry_template_name:
        template_name = layout.add_dashboard_entry_template_name

    if versions.DJANGO_GTE_1_10:
        return render(request, template_name, context)
    else:
        return render_to_response(
            template_name, context, context_instance=RequestContext(request)
        )


@login_required
@permission_required('dash.change_dashboardentry')
def edit_dashboard_entry(request,
                         entry_id,
                         template_name='dash/edit_dashboard_entry.html',
                         template_name_ajax='dash/edit_dashboard_entry_ajax'
                                            '.html'):
    """Edit dashboard entry.

    :param django.http.HttpRequest request:
    :param int entry_id: ID of the dashboard entry to edit.
    :param string template_name:
    :param string template_name_ajax: Template used for AJAX requests.
    :return django.http.HttpResponse:
    """
    # Getting dashboard settings for the user. Then get users' layout.
    dashboard_settings = get_or_create_dashboard_settings(request.user)

    try:
        obj = DashboardEntry._default_manager \
                            .select_related('workspace') \
                            .get(pk=entry_id, user=request.user)
    except ObjectDoesNotExist as err:
        raise Http404(err)

    if obj.layout_uid:
        layout_uid = obj.layout_uid
    else:
        layout_uid = dashboard_settings.layout_uid

    layout = get_layout(
        layout_uid=layout_uid, as_instance=True
    )

    plugin = obj.get_plugin(fetch_related_data=True)
    plugin.request = request

    if plugin.edit_form_template:
        template_name = plugin.edit_form_template

    # Template context
    context = {
        'layout': layout,
        'dashboard_settings': dashboard_settings
    }

    # If plugin has form, it is configurable which means we have to load the
    # plugin form and validate user input.
    plugin_form = plugin.get_form()
    if plugin_form:
        # If POST request and form data is valid, save the data and redirect
        # to the dashboard edit.
        if request.method == 'POST':
            form = plugin.get_initialised_edit_form_or_404(
                data=request.POST,
                files=request.FILES,
            )
            if form.is_valid():
                # Saving the plugin form data.
                form.save_plugin_data(request=request)

                # Getting the plugin data.
                obj.plugin_data = form.get_plugin_data(request=request)

                # Save the object.
                obj.save()

                messages.info(
                    request,
                    _('The dashboard widget "{0}" was edited '
                      'successfully.').format(plugin.name)
                )

                # Redirect to edit dashboard view
                if obj.workspace:
                    return redirect(
                        'dash.edit_dashboard', workspace=obj.workspace.slug
                    )
                else:
                    return redirect('dash.edit_dashboard')

        else:
            form = plugin.get_initialised_edit_form_or_404()

        context.update({'form': form, 'plugin': plugin})

    if layout.edit_dashboard_entry_ajax_template_name:
        template_name_ajax = layout.edit_dashboard_entry_ajax_template_name

    context.update(
        {'edit_dashboard_entry_ajax_template_name': template_name_ajax}
    )

    if request.is_ajax():
        template_name = template_name_ajax
    elif layout.edit_dashboard_entry_template_name:
        template_name = layout.edit_dashboard_entry_template_name

    if versions.DJANGO_GTE_1_10:
        return render(request, template_name, context)
    else:
        return render_to_response(
            template_name, context, context_instance=RequestContext(request)
        )


@login_required
@permission_required('dash.delete_dashboardentry')
def delete_dashboard_entry(request, entry_id):
    """Remove dashboard entry.

    :param django.http.HttpRequest request:
    :param int entry_id: ID of the dashboard entry to delete.
    :return django.http.HttpResponse:
    """
    try:
        obj = DashboardEntry._default_manager \
                            .select_related('workspace') \
                            .get(pk=entry_id, user=request.user)
        plugin = obj.get_plugin()
        plugin.request = request
        plugin._delete_plugin_data()
        workspace = getattr(obj.workspace, 'slug', None)
        obj.delete()

        if not request.is_ajax():
            messages.info(request, _('The dashboard widget "{0}" was deleted '
                                     'successfully.').format(plugin.name))

        if request.is_ajax():
            return HttpResponse(json.dumps({'success': 1}))
        else:
            # Redirect to dashboard view.
            if workspace:
                return redirect('dash.edit_dashboard', workspace=workspace)
            else:
                return redirect('dash.edit_dashboard')
    except ObjectDoesNotExist as err:
        if request.is_ajax():
            return HttpResponse(json.dumps({'success': 1}))
        raise Http404(err)

# ***************************************************************************
# ***************************************************************************
# ************************** Dashboard plugins  *****************************
# ***************************************************************************
# ***************************************************************************


@login_required
@permission_required('dash.add_dashboardentry')
def plugin_widgets(request,
                   placeholder_uid,
                   workspace=None,
                   position=None,
                   template_name='dash/plugin_widgets.html',
                   template_name_ajax='dash/plugin_widgets_ajax.html'):
    """Plugin widgets view.

    Lists all the widgets for the placeholder and workspace given.

    :param django.http.HttpRequest request:
    :param string placeholder_uid: Placeholder UID.
    :param mixed workspace:
    :param int position: Position on the dashboard to which the widget is to
        be added.
    :param string template_name:
    :param string template_name_ajax: Template used for AJAX requests.
    :return django.http.HttpResponse:
    """
    # Getting dashboard settings for the user. Then get users' layout.
    dashboard_settings = get_or_create_dashboard_settings(request.user)
    layout = get_layout(
        layout_uid=dashboard_settings.layout_uid,
        as_instance=True
    )

    placeholder = layout.get_placeholder(placeholder_uid)

    if not validate_placeholder_uid(layout, placeholder_uid):
        raise Http404(
            ugettext("Invalid placeholder: {0}").format(placeholder_uid)
        )

    occupied_cells = build_cells_matrix(
        request.user,
        layout,
        placeholder,
        workspace=workspace
    )

    # Here we checking if clipboard contains a plugin which is suitable for
    # being pasted into the cell given.
    paste_from_clipboard_url = None

    # First get the clipboard data.
    clipboard_plugin_data = get_plugin_data_from_clipboard(request, layout.uid)

    # If clipboard data is not empty, check if the data is suitable for
    # being pasted into the position given.
    if clipboard_plugin_data:
        can_paste_from_clipboard = can_paste_entry_from_clipboard(
            request=request,
            layout=layout,
            placeholder_uid=placeholder_uid,
            position=position,
            workspace=workspace
        )

        if can_paste_from_clipboard:
            kwargs = {
                'placeholder_uid': placeholder_uid,
                'position': position
            }
            if workspace:
                kwargs.update({'workspace': workspace})

            paste_from_clipboard_url = reverse(
                'dash.paste_dashboard_entry',
                kwargs=kwargs
            )

    context = {
        'layout': layout,
        'grouped_widgets': get_widgets(
            layout,
            placeholder,
            request.user,
            workspace=workspace,
            position=position,
            occupied_cells=occupied_cells
        ),
        'dashboard_settings': dashboard_settings,
        'paste_from_clipboard_url': paste_from_clipboard_url,
    }

    if request.is_ajax():
        template_name = layout.plugin_widgets_template_name_ajax

    if versions.DJANGO_GTE_1_10:
        return render(request, template_name, context)
    else:
        return render_to_response(
            template_name, context, context_instance=RequestContext(request)
        )

# ***************************************************************************
# ***************************************************************************
# **************** Create/edit/delete dashboard workspaces ******************
# ***************************************************************************
# ***************************************************************************


@login_required
@permission_required('dash.add_dashboardworkspace')
def create_dashboard_workspace(request,
                               template_name='dash/create_dashboard_workspace'
                                             '.html',
                               template_name_ajax='dash/create_dashboard_'
                                                  'workspace_ajax.html'):
    """Create dashboard workspace.

    :param django.http.HttpRequest request:
    :param string template_name:
    :param string template_name_ajax: Template used for AJAX calls.
    :return django.http.HttpResponse:
    """
    # Getting dashboard settings for the user. Then get users' layout.
    dashboard_settings = get_or_create_dashboard_settings(request.user)
    layout = get_layout(
        layout_uid=dashboard_settings.layout_uid, as_instance=True
    )

    if request.method == 'POST':
        form = DashboardWorkspaceForm(
            data=request.POST,
            files=request.FILES,
            different_layouts=dashboard_settings.allow_different_layouts
        )
        if form.is_valid():
            obj = form.save(commit=False)
            obj.user = request.user
            if not dashboard_settings.allow_different_layouts:
                obj.layout_uid = layout.uid
            obj.save()
            messages.info(
                request,
                _('The dashboard workspace "{0}" was '
                  'created successfully.').format(obj.name)
            )
            return redirect('dash.edit_dashboard', workspace=obj.slug)

    else:
        form = DashboardWorkspaceForm(
            initial={
                'user': request.user,
                'layout_uid': layout.uid,
            },
            different_layouts=dashboard_settings.allow_different_layouts
        )

    if layout.create_dashboard_workspace_ajax_template_name:
        template_name_ajax = \
            layout.create_dashboard_workspace_ajax_template_name

    if request.is_ajax():
        template_name = template_name_ajax
    elif layout.create_dashboard_workspace_template_name:
        template_name = layout.create_dashboard_workspace_template_name

    context = {
        'layout': layout,
        'form': form,
        'dashboard_settings': dashboard_settings
    }

    if versions.DJANGO_GTE_1_10:
        return render(request, template_name, context)
    else:
        return render_to_response(
            template_name, context, context_instance=RequestContext(request)
        )


@login_required
@permission_required('dash.change_dashboardworkspace')
def edit_dashboard_workspace(request, workspace_id,
                             template_name='dash/edit_dashboard_workspace'
                                           '.html',
                             template_name_ajax='dash/edit_dashboard_'
                                                'workspace_ajax.html'):
    """Edit dashboard workspace.

    :param django.http.HttpRequest request:
    :param int workspace_id: DashboardWorkspace ID.
    :param string template_name:
    :param string template_name_ajax: Template used for AJAX calls.
    :return django.http.HttpResponse:
    """
    # Getting dashboard settings for the user. Then get users' layout.
    dashboard_settings = get_or_create_dashboard_settings(request.user)

    # Check if user trying to edit the dashboard workspace actually owns it.
    try:
        obj = DashboardWorkspace._default_manager \
            .get(pk=workspace_id, user=request.user)
    except ObjectDoesNotExist as err:
        raise Http404(err)

    layout = get_layout(
        layout_uid=obj.layout_uid, as_instance=True
    )

    if request.method == 'POST':
        form = DashboardWorkspaceForm(
            data=request.POST,
            files=request.FILES,
            instance=obj,
            different_layouts=dashboard_settings.allow_different_layouts
        )
        if form.is_valid():
            form.save(commit=False)
            obj.user = request.user
            if not dashboard_settings.allow_different_layouts:
                obj.layout_uid = layout.uid
            obj.save()
            messages.info(
                request,
                _('The dashboard workspace "{0}" was '
                  'edited successfully.').format(obj.name))
            return redirect('dash.edit_dashboard', workspace=obj.slug)

    else:
        form = DashboardWorkspaceForm(
            instance=obj,
            different_layouts=dashboard_settings.allow_different_layouts
        )

    if layout.edit_dashboard_workspace_ajax_template_name:
        template_name_ajax = layout.edit_dashboard_workspace_ajax_template_name

    if request.is_ajax():
        template_name = template_name_ajax
    elif layout.edit_dashboard_workspace_template_name:
        template_name = layout.edit_dashboard_workspace_template_name

    context = {
        'layout': layout,
        'form': form,
        'workspace_id': workspace_id,
        'dashboard_settings': dashboard_settings
    }

    if versions.DJANGO_GTE_1_10:
        return render(request, template_name, context)
    else:
        return render_to_response(
            template_name, context, context_instance=RequestContext(request)
        )


@login_required
@permission_required('dash.delete_dashboardworkspace')
def delete_dashboard_workspace(request, workspace_id,
                               template_name='dash/delete_dashboard_workspace'
                                             '.html',
                               template_name_ajax='dash/delete_dashboard_'
                                                  'workspace_ajax.html'):
    """Delete dashboard workspace.

    :param django.http.HttpRequest request:
    :param int workspace_id: DashboardWorkspace id.
    :param string template_name:
    :param string template_name_ajax: Template used for AJAX calls.
    :return django.http.HttpResponse:
    """
    # Getting dashboard settings for the user. Then get users' layout.
    dashboard_settings = get_or_create_dashboard_settings(request.user)

    # Check if user trying to edit the dashboard workspace actually owns it
    # and then delete the workspace.
    if (
            request.method == 'POST'
            and 'delete' in request.POST is None
            and request.POST.get('next', None)
    ):
        return redirect(request.POST.get('next'))

    try:
        obj = DashboardWorkspace._default_manager \
                                .get(pk=workspace_id, user=request.user)

    except ObjectDoesNotExist as err:
        raise Http404(err)

    layout = get_layout(
        layout_uid=workspace.layout_uid, as_instance=True
        )

    if request.method == 'POST':
        if 'delete' in request.POST:
            workspace_name = obj.name

            # Getting the (frozen) queryset.
            dashboard_entries = DashboardEntry._default_manager \
                .filter(user=request.user,
                        layout_uid=layout.uid,
                        workspace__id=workspace_id) \
                .select_related('workspace', 'user') \
                .order_by('placeholder_uid', 'position')[:]

            # Cleaning the plugin data for the deleted entries.
            clean_plugin_data(dashboard_entries, request=request)

            # Delete the workspace.
            obj.delete()

            messages.info(
                request,
                _('The dashboard workspace "{0}" was deleted '
                  'successfully.').format(workspace_name)
            )
            return redirect('dash.edit_dashboard')

        if request.POST.get('next', None):
            return redirect(request.POST.get('next'))

    if request.is_ajax():
        template_name = template_name_ajax

    context = {
        'layout': layout,
        'workspace': obj,
        'dashboard_settings': dashboard_settings
    }

    if versions.DJANGO_GTE_1_10:
        return render(request, template_name, context)
    else:
        return render_to_response(
            template_name, context, context_instance=RequestContext(request)
        )


@login_required
def dashboard_workspaces(request,
                         workspace=None,
                         template_name='dash/dashboard_workspaces.html',
                         template_name_ajax='dash/dashboard_workspaces_ajax'
                                            '.html'):
    """Workspaces list.

    :param django.http.HttpRequest request:
    :param string workspace: Workspace slug.
    :param string template_name:
    :param string template_name_ajax: Tempalte used for AJAX requests.
    :return django.http.HttpResponse:
    """
    # Getting dashboard settings for the user. Then get users' layout.
    dashboard_settings = get_or_create_dashboard_settings(request.user)

    context = get_workspaces(
        request.user,
        dashboard_settings.layout_uid,
        workspace,
        different_layouts=dashboard_settings.allow_different_layouts
    )

    if dashboard_settings.allow_different_layouts \
            and context['current_workspace']:
        layout_uid = context['current_workspace'].layout_uid
    else:
        layout_uid = dashboard_settings.layout_uid

    layout = get_layout(
        layout_uid=layout_uid,
        as_instance=True
    )

    context.update({
        'layout': layout,
        'dashboard_settings': dashboard_settings
    })

    if request.is_ajax():
        template_name = template_name_ajax

    if versions.DJANGO_GTE_1_10:
        return render(request, template_name, context)
    else:
        return render_to_response(
            template_name, context, context_instance=RequestContext(request)
        )

# ***************************************************************************
# ***************************************************************************
# ************************* Dashboard settings ******************************
# ***************************************************************************
# ***************************************************************************


@login_required
@permission_required('dash.change_dashboardsettings')
def edit_dashboard_settings(request,
                            template_name='dash/edit_dashboard_settings.html',
                            template_name_ajax='dash/edit_dashboard_settings_'
                                               'ajax.html'):
    """Edit dashboard settings.

    :param django.http.HttpRequest request:
    :param string template_name:
    :param string template_name_ajax: Template used for AJAX calls.
    :return django.http.HttpResponse:
    """
    # Getting dashboard settings for the user. Then get users' layout.
    dashboard_settings = get_or_create_dashboard_settings(request.user)
    layout = get_layout(
        layout_uid=dashboard_settings.layout_uid, as_instance=True
    )

    if request.method == 'POST':
        form = DashboardSettingsForm(
            data=request.POST,
            files=request.FILES,
            instance=dashboard_settings
        )
        if form.is_valid():
            form.save(commit=False)
            dashboard_settings.user = request.user
            dashboard_settings.save()
            messages.info(request, _('Dashboard settings were edited '
                                     'successfully.'))
            return redirect('dash.edit_dashboard')

    else:
        form = DashboardSettingsForm(instance=dashboard_settings)

    if request.is_ajax():
        template_name = template_name_ajax

    context = {
        'layout': layout,
        'form': form,
        'dashboard_settings': dashboard_settings
    }

    if versions.DJANGO_GTE_1_10:
        return render(request, template_name, context)
    else:
        return render_to_response(
            template_name, context, context_instance=RequestContext(request)
        )


@login_required
@permissions_required(satisfy=SATISFY_ALL,
                      perms=['dash.add_dashboardentry',
                             'dash.add_dashboardworkspace'])
def clone_dashboard_workspace(request, workspace_id):
    """Clones dashboard workspace."""
    redirect_to = request.GET.get('next', None)

    try:
        workspace = DashboardWorkspace._default_manager.get(pk=workspace_id)
    except DashboardWorkspace.DoesNotExist:
        messages.info(request, _("Invalid dashboard workspace."))
        if redirect_to:
            return redirect(redirect_to)
        else:
            return redirect('dash.edit_dashboard')

    if not (workspace.is_clonable or request.user.pk == workspace.user.pk):
        messages.info(request, _("You are not allowed to clone the given "
                                 "workspace."))
        if redirect_to:
            return redirect(redirect_to)
        else:
            return redirect('dash.edit_dashboard')

    cloned_workspace = clone_workspace(workspace, request.user)

    # Getting dashboard settings for the user. Then get users' layout.
    dashboard_settings = get_or_create_dashboard_settings(request.user)
    cloned_workspace_layout = get_layout(
        layout_uid=workspace.layout_uid, as_instance=True
    )
    layout = get_layout(
        layout_uid=dashboard_settings.layout_uid, as_instance=True
    )

    if workspace.layout_uid == layout.uid or \
            dashboard_settings.allow_different_layouts:

        messages.info(
            request,
            _("Dashboard workspace `{0}` was successfully cloned into "
              "`{1}`.".format(workspace.name, cloned_workspace.name))
        )
        return redirect('dash.edit_dashboard', workspace=cloned_workspace.slug)

    else:

        messages.info(
            request,
            _("Dashboard workspace `{0}` was successfully cloned into `{1}` "
              "(layout `{2}`), however your active layout is `{3}`. You "
              "should switch to layout `{4}` (in your dashboard settings) "
              "in order to see the cloned workspace."
              "".format(workspace.name,
                        cloned_workspace.name,
                        cloned_workspace_layout.name,
                        layout.name,
                        cloned_workspace_layout.name))
        )
        return redirect('dash.edit_dashboard')

# ***************************************************************************
# ***************************************************************************
# **************************** Clipboard views ******************************
# ***************************************************************************
# ***************************************************************************


@login_required
@use_clipboard_permission_required(
    raise_exception=RAISE_EXCEPTION_WHEN_PERMISSIONS_INSUFFICIENT
)
def cut_dashboard_entry(request, entry_id):
    """Cut the given dashboard entry.

    It's not possible to remove undeletable entries.

    :param django.http.HttpRequest request:
    :param int entry_id: ID of the DashboardEntry to cut to clipboard.
    :return django.http.HttpResponse:
    """
    dashboard_entry = DashboardEntry._default_manager \
                                    .select_related('workspace') \
                                    .get(pk=entry_id, user=request.user)
    workspace = dashboard_entry.workspace
    plugin = dashboard_entry.get_plugin()

    cut_entry_to_clipboard(request, dashboard_entry)

    messages.info(request, _('The dashboard function "{0}" was successfully '
                             'cut and placed into the '
                             'clipboard.').format(safe_text(plugin.name)))

    if workspace and workspace.slug:
        return redirect('dash.edit_dashboard', workspace=workspace.slug)
    else:
        return redirect('dash.edit_dashboard')


@login_required
@use_clipboard_permission_required(
    raise_exception=RAISE_EXCEPTION_WHEN_PERMISSIONS_INSUFFICIENT
)
def copy_dashboard_entry(request, entry_id):
    """Cut the given dashboard entry.

    It's not possible to remove undeletable entries.

    :param django.http.HttpRequest request:
    :param int entry_id: ID of the DashboardEntry to cut to clipboard.
    :return django.http.HttpResponse:
    """
    # Getting dashboard settings for the user. Then get users' layout.
    # dashboard_settings = get_or_create_dashboard_settings(request.user)
    # layout = get_layout(
    #     layout_uid=dashboard_settings.layout_uid, as_instance=True
    # )

    dashboard_entry = DashboardEntry._default_manager \
                                    .select_related('workspace') \
                                    .get(pk=entry_id, user=request.user)
    workspace = dashboard_entry.workspace
    plugin = dashboard_entry.get_plugin()

    copy_entry_to_clipboard(request, dashboard_entry)

    messages.info(request, _('The dashboard entry "{0}" was successfully '
                             'copied and placed into the '
                             'clipboard.').format(safe_text(plugin.name)))

    if workspace and workspace.slug:
        return redirect('dash.edit_dashboard', workspace=workspace.slug)
    else:
        return redirect('dash.edit_dashboard')


@login_required
@use_clipboard_permission_required(
    raise_exception=RAISE_EXCEPTION_WHEN_PERMISSIONS_INSUFFICIENT
)
def paste_dashboard_entry(request, placeholder_uid, position, workspace=None):
    """Paste the dashboard entry from clipboard if any available.

    :param django.http.HttpRequest request:
    :param str placeholder_uid:
    :param int position:
    :param str workspace: Workspace slug.
    :return django.http.HttpResponse:
    """
    # Getting dashboard settings for the user. Then get users' layout.
    dashboard_settings = get_or_create_dashboard_settings(request.user)
    layout = get_layout(
        layout_uid=dashboard_settings.layout_uid,
        as_instance=True
    )

    try:
        plugin_uid, success = paste_entry_from_clipboard(
            request,
            layout,
            placeholder_uid,
            position,
            workspace=workspace
        )
    except Exception as err:
        plugin_uid, success = str(err), False

    if plugin_uid and success:
        plugin = plugin_registry.get(plugin_uid)
        messages.info(
            request,
            _('The dashboard entry "{0}" was successfully pasted from '
              'clipboard.').format(safe_text(plugin.name))
        )
    else:
        # In case if not success, ``plugin_uid`` would be holding the error
        # message.
        messages.info(
            request,
            _('Problems occurred while pasting from '
              'clipboard. {0}'.format(safe_text(plugin_uid)))
        )

    if workspace:
        return redirect('dash.edit_dashboard', workspace=workspace)
    else:
        return redirect('dash.edit_dashboard')



from ....contrib.plugins.url.forms import URLForm

from .settings import IMAGE_CHOICES_WITH_EMPTY_OPTION

__title__ = 'dash.contrib.layouts.bootstrap3.dash_widgets'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2017 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('URLBootstrapThreeForm',)


class URLBootstrapThreeForm(URLForm):
    """URL bootstrap 3 form.

    Almost like the original, but has less options.
    """
    def __init__(self, *args, **kwargs):
        super(URLBootstrapThreeForm, self).__init__(*args, **kwargs)
        self.fields['image'].choices = IMAGE_CHOICES_WITH_EMPTY_OPTION



import datetime
from time import struct_time

from django import template

__title__ = 'dash.contrib.plugins.rss_feed.templatetags.rss_feed_tags'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2017 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('convert_to_datetime',)

register = template.Library()


@register.filter
def convert_to_datetime(value):
    """Convert to datetime."""

    if isinstance(value, struct_time):
        converted = datetime.datetime(
            year=value.tm_year,
            month=value.tm_mon,
            day=value.tm_mday,
            hour=value.tm_hour,
            minute=value.tm_min,
            second=value.tm_sec
        )
    else:
        converted = value
    return converted



from django.template.loader import render_to_string

from ....base import BaseDashboardPluginWidget

__title__ = 'dash.contrib.plugins.video.dash_widgets'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2017 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'BaseVideoWidget',
    'Video1x1Widget',
    'Video2x2Widget',
    'Video3x3Widget',
    'Video4x4Widget',
    'Video5x5Widget'
)

# **********************************************************************
# *********************** Base Video widget plugin *********************
# **********************************************************************


class BaseVideoWidget(BaseDashboardPluginWidget):
    """Base video plugin widget."""

    media_css = (
        'css/dash_plugin_video.css',
    )

    def render(self, request=None):
        """Render."""
        context = {'plugin': self.plugin}
        return render_to_string('video/render.html', context)

# **********************************************************************
# ************************** Specific widgets **************************
# **********************************************************************


class Video1x1Widget(BaseVideoWidget):
    """Video plugin 1x1 widget."""

    plugin_uid = 'video_1x1'


class Video2x2Widget(BaseVideoWidget):
    """Video plugin 2x2 widget."""

    plugin_uid = 'video_2x2'
    cols = 2
    rows = 2


class Video3x3Widget(BaseVideoWidget):
    """Video plugin 3x3 widget."""

    plugin_uid = 'video_3x3'
    cols = 3
    rows = 3


class Video4x4Widget(BaseVideoWidget):
    """Video plugin 4x4 widget."""

    plugin_uid = 'video_4x4'
    cols = 4
    rows = 4


class Video5x5Widget(BaseVideoWidget):
    """Video plugin 5x5 widget."""

    plugin_uid = 'video_5x5'
    cols = 5
    rows = 5



from django import forms
from django.utils.translation import ugettext_lazy as _

from ....base import DashboardPluginFormBase
from ....widgets import BooleanRadioSelect

from .defaults import (
    DEFAULT_CACHE_FOR,
    DEFAULT_MAX_FEED_ITEMS,
    DEFAULT_SHOW_TITLE,
    DEFAULT_TRUNCATE_AFTER,
)

__title__ = 'dash.contrib.plugins.rss_feed.forms'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2017 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('ReadRSSFeedForm',)


class ReadRSSFeedForm(forms.Form, DashboardPluginFormBase):
    """Form for main ``ReadRSSFeedPlugin``."""

    plugin_data_fields = [
        ("feed_url", ""),
        ("custom_feed_title", ""),
        ("show_feed_title", DEFAULT_SHOW_TITLE),
        ("max_items", DEFAULT_MAX_FEED_ITEMS),
        ("truncate_after", DEFAULT_TRUNCATE_AFTER),
        ("cache_for", DEFAULT_CACHE_FOR)
    ]
    feed_url = forms.URLField(label=_("Feed URL"), required=True)
    custom_feed_title = forms.CharField(label=_("Custom feed title"),
                                        required=False)
    show_feed_title = forms.BooleanField(label=_("Show feed title?"),
                                         required=False,
                                         initial=DEFAULT_SHOW_TITLE,
                                         widget=BooleanRadioSelect)
    max_items = forms.IntegerField(label=_("Max feed items to show"),
                                   required=True,
                                   initial=DEFAULT_MAX_FEED_ITEMS)
    truncate_after = forms.IntegerField(label=_("Truncate after"),
                                        required=False,
                                        initial=DEFAULT_TRUNCATE_AFTER)
    cache_for = forms.IntegerField(label=_("Cache for"),
                                   required=True,
                                   initial=DEFAULT_CACHE_FOR)



"""
Compatibility module for safe and sane User model import.
"""

from django.core.exceptions import ImproperlyConfigured

from nine.user import User

__title__ = 'dash.compat'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2017 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('User',)


# Sanity checks. Possibly rely on the dynamic username field in future.
user = User()

if not hasattr(user, 'username'):
    from .exceptions import ImproperlyConfigured
    raise ImproperlyConfigured(
        "Your custom user model ({0}.{1}) doesn't have ``username`` "
        "property, while ``django-dash`` relies on its' presence."
        "".format(user._meta.app_label, user._meta.object_name)
    )



import logging

from django.contrib import admin
from django.utils.translation import ugettext_lazy as _

from ....models import DashboardEntry
from ....utils import update_plugin_data_for_entries

from .models import Bookmark

__title__ = 'dash.contrib.plugins.urls.admin'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2017 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('BookmarkAdmin',)

logger = logging.getLogger(__name__)


class BookmarkAdmin(admin.ModelAdmin):
    """Bookmark admin."""

    list_display = ('title', 'url', 'external', 'image')
    fieldsets = (
        (None, {
            'fields': ('title', 'url', 'external', 'image')
        }),
    )

    class Meta(object):
        """Meta."""

        app_label = _('Bookmark')

    def save_model(self, request, obj, form, change):
        """Updating the bookmark plugin entries on bookmarks update."""
        super(BookmarkAdmin, self).save_model(request, obj, form, change)

        dashboard_entries = DashboardEntry._default_manager.filter(
            plugin_uid__startswith='bookmark_'
        )
        try:
            update_plugin_data_for_entries(
                dashboard_entries=dashboard_entries,
                request=request
            )
        except Exception as err:
            logger.debug(str(err))


admin.site.register(Bookmark, BookmarkAdmin)



from ....base import (
    BaseDashboardLayout,
    BaseDashboardPlaceholder,
    layout_registry
)

__title__ = 'dash.contrib.layouts.android.dash_layouts'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2017 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('AndroidLayout',)


class AndroidMainPlaceholder(BaseDashboardPlaceholder):
    """Main placeholder."""

    uid = 'main'
    cols = 6
    rows = 5
    cell_width = 150
    cell_height = 110


class AndroidShortcutsPlaceholder(BaseDashboardPlaceholder):
    """Shortcuts placeholder."""

    uid = 'shortcuts'
    cols = 1
    rows = 10
    cell_width = 60
    cell_height = 55


class AndroidLayout(BaseDashboardLayout):
    """Android layout."""

    uid = 'android'
    name = 'Android'
    view_template_name = 'android/view_layout.html'
    edit_template_name = 'android/edit_layout.html'
    placeholders = [AndroidMainPlaceholder, AndroidShortcutsPlaceholder]
    cell_units = 'px'
    media_css = (
        'css/dash_dotted_borders.css',
        'android/css/dash_layout_android.css',
    )
    # media_js = ('js/dash_layout_android.js',)


layout_registry.register(AndroidLayout)



__title__ = 'dash.contrib.plugins.dummy'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2017 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'default_app_config',
    'UID',
)

default_app_config = 'dash.contrib.plugins.dummy.apps.Config'

UID = 'dummy'



from ....contrib.plugins.url.dash_widgets import (
    BaseBookmarkWidget,
    BaseURLWidget,
)

__title__ = 'dash.contrib.layouts.bootstrap2.dash_widgets'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2017 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'BaseBookmarkBootstrapTwoWidget',
    'URLBootstrapTwo1x1Bootstrap2FluidMainWidget',
    'URLBootstrapTwo2x2Bootstrap2FluidMainWidget',
)

# *************************************************************
# ******************* URL widgets *****************************
# *************************************************************


class URLBootstrapTwo1x1Bootstrap2FluidMainWidget(BaseURLWidget):
    """URL plugin 1x1 widget for Bootstrap 2 Fluid layout.

    Placeholder `main` widget.
    """

    layout_uid = 'bootstrap2_fluid'
    placeholder_uid = 'main'
    plugin_uid = 'url_bootstrap_two_1x1'
    media_css = (
        'css/dash_plugin_url_bootstrap2.css',
    )


class URLBootstrapTwo2x2Bootstrap2FluidMainWidget(
    URLBootstrapTwo1x1Bootstrap2FluidMainWidget
):
    """URL2x2 plugin widget for Bootstrap 2 Fluid layout.

    Placeholder `main` widget.
    """

    plugin_uid = 'url_bootstrap_two_2x2'
    cols = 2
    rows = 2

# *********************************************************
# *********************************************************
# *********************** Bookmark widgets ****************
# *********************************************************
# *********************************************************


class BaseBookmarkBootstrapTwoWidget(BaseBookmarkWidget):
    """Base Bookmark plugin widget for Bootstrap 2 Fluid layout."""

    media_css = (
        'css/dash_plugin_bookmark_bootstrap2.css',
    )



__title__ = 'dash.exceptions'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2017 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'BaseException',
    'ImproperlyConfigured',
    'InvalidRegistryItemType',
    'LayoutDoesNotExist',
    'NoActiveLayout',
    'PluginWidgetOutOfPlaceholderBoundaries',
)


class BaseException(Exception):
    """Base django-dash exception."""


class InvalidRegistryItemType(ValueError):
    """Raised when an attempt is made to register an item of improper type.

    Raised when an attempt is made to register an item of improper type in
    the registry .
    """


class LayoutDoesNotExist(BaseException):
    """Raised when layout does not exist."""


class NoActiveLayoutChosen(BaseException):
    """Raised when no active layout is chosen."""


class PluginWidgetOutOfPlaceholderBoundaries(BaseException):
    """Raised when plugin widget is out of placeholder boundaries."""


class ImproperlyConfigured(BaseException):
    """Raised when ``django-dash`` is somehow improperly configured."""



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import autoslug.fields
import dash.fields
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('auth', '__latest__'),
    ]

    operations = [
        migrations.CreateModel(
            name='DashboardEntry',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('layout_uid', models.CharField(max_length=25, verbose_name='Layout')),
                ('placeholder_uid', models.CharField(max_length=255, verbose_name='Placeholder')),
                ('plugin_uid', models.CharField(max_length=255, verbose_name='Plugin name')),
                ('plugin_data', models.TextField(null=True, verbose_name='Plugin data', blank=True)),
                ('position', models.PositiveIntegerField(null=True, verbose_name='Position', blank=True)),
                ('user', models.ForeignKey(verbose_name='User', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'Dashboard entry',
                'verbose_name_plural': 'Dashboard entries',
            },
        ),
        migrations.CreateModel(
            name='DashboardPlugin',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('plugin_uid', models.CharField(verbose_name='Plugin UID', unique=True, max_length=255, editable=False)),
                ('groups', models.ManyToManyField(to='auth.Group', null=True, verbose_name='Group', blank=True)),
                ('users', models.ManyToManyField(to=settings.AUTH_USER_MODEL, null=True, verbose_name='User', blank=True)),
            ],
            options={
                'verbose_name': 'Dashboard plugin',
                'verbose_name_plural': 'Dashboard plugins',
            },
        ),
        migrations.CreateModel(
            name='DashboardSettings',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('layout_uid', models.CharField(max_length=25, verbose_name='Layout')),
                ('title', models.CharField(max_length=255, verbose_name='Title')),
                ('is_public', models.BooleanField(default=False, help_text='Makes your dashboard to be visible to the public. Visibility of workspaces could be adjust separately for each workspace, however setting your dashboard to be visible to public, makes your default workspace visible to public too.', verbose_name='Is public?')),
                ('user', models.ForeignKey(verbose_name='User', to=settings.AUTH_USER_MODEL, unique=True)),
            ],
            options={
                'verbose_name': 'Dashboard settings',
                'verbose_name_plural': 'Dashboard settings',
            },
        ),
        migrations.CreateModel(
            name='DashboardWorkspace',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('layout_uid', models.CharField(max_length=25, verbose_name='Layout')),
                ('name', models.CharField(max_length=255, verbose_name='Name')),
                ('slug', autoslug.fields.AutoSlugField(verbose_name='Slug', unique=True, editable=False)),
                ('position', dash.fields.OrderField(null=True, verbose_name='Position', blank=True)),
                ('is_public', models.BooleanField(default=False, help_text='Makes your workspace to be visible to the public.', verbose_name='Is public?')),
                ('is_clonable', models.BooleanField(default=False, help_text='Makes your workspace to be cloneable by other users.', verbose_name='Is cloneable?')),
                ('user', models.ForeignKey(verbose_name='User', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'Dashboard workspace',
                'verbose_name_plural': 'Dashboard workspaces',
            },
        ),
        migrations.AddField(
            model_name='dashboardentry',
            name='workspace',
            field=models.ForeignKey(verbose_name='Workspace', blank=True, to='dash.DashboardWorkspace', null=True),
        ),
        migrations.AlterUniqueTogether(
            name='dashboardworkspace',
            unique_together=set([('user', 'slug'), ('user', 'name')]),
        ),
    ]



from django.conf.urls import url

from .views import get_feed

__title__ = 'dash.contrib.plugins.rss_feed.urls'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2017 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('urlpatterns',)

urlpatterns = [
    url(r'^get_feed/(?P<layout_uid>[\w_]+)/(?P<placeholder_uid>[\w_]+)/'
        r'(?P<plugin_uid>[\w_\-]+)/$',
        view=get_feed,
        name='dash.contrib.plugins.rss_feed.get_feed'),
]



from django.utils.translation import ugettext_lazy as _

from ....base import BaseDashboardPlugin
from ....factory import plugin_factory

from .forms import ReadRSSFeedForm

__title__ = 'dash.contrib.plugins.rss_feed.dash_plugins'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2017 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('BaseReadRSSFeedPlugin',)

# ****************************************************************************
# ********************************* Base Read RSS feed plugin ****************
# ****************************************************************************


class BaseReadRSSFeedPlugin(BaseDashboardPlugin):
    """Base Read RSS feed into HTML plugin."""

    name = _("Read RSS feed")
    form = ReadRSSFeedForm
    group = _("Internet")

# ****************************************************************************
# ********** Generating and registering the plugins using factory ************
# ****************************************************************************


sizes = (
    (2, 3),
    (3, 3),
)

plugin_factory(BaseReadRSSFeedPlugin, 'read_rss_feed', sizes)



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

from dash.compat import User

# With the default User model these will be 'auth.User' and 'auth.user'
# so instead of using orm['auth.User'] we can use orm[user_orm_label]
user_orm_label = '%s.%s' % (User._meta.app_label, User._meta.object_name)
user_model_label = '%s.%s' % (User._meta.app_label, User._meta.module_name)

class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding field 'DashboardWorkspace.is_clonable'
        db.add_column(u'dash_dashboardworkspace', 'is_clonable',
                      self.gf('django.db.models.fields.BooleanField')(default=False),
                      keep_default=False)


    def backwards(self, orm):
        # Deleting field 'DashboardWorkspace.is_clonable'
        db.delete_column(u'dash_dashboardworkspace', 'is_clonable')


    models = {
        u'auth.group': {
            'Meta': {'object_name': 'Group'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        u'auth.permission': {
            'Meta': {'ordering': "(u'content_type__app_label', u'content_type__model', u'codename')", 'unique_together': "((u'content_type', u'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        user_model_label: {
            'Meta': {'object_name': User.__name__, 'db_table': "'%s'" % User._meta.db_table},
            User._meta.pk.attname: ('django.db.models.fields.AutoField', [], {'primary_key': 'True', 'db_column': "'%s'" % User._meta.pk.column}),
        },
        u'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        u'dash.dashboardentry': {
            'Meta': {'object_name': 'DashboardEntry'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'layout_uid': ('django.db.models.fields.CharField', [], {'max_length': '25'}),
            'placeholder_uid': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'plugin_data': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'plugin_uid': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'position': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm[user_orm_label]"}),
            'workspace': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['dash.DashboardWorkspace']", 'null': 'True', 'blank': 'True'})
        },
        u'dash.dashboardplugin': {
            'Meta': {'object_name': 'DashboardPlugin'},
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'to': u"orm['auth.Group']", 'null': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'plugin_uid': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '255'}),
            'users': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'to': u"orm[user_orm_label]", 'null': 'True', 'blank': 'True'})
        },
        u'dash.dashboardsettings': {
            'Meta': {'object_name': 'DashboardSettings'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_public': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'layout_uid': ('django.db.models.fields.CharField', [], {'max_length': '25'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm[user_orm_label]", 'unique': 'True'})
        },
        u'dash.dashboardworkspace': {
            'Meta': {'unique_together': "(('user', 'slug'), ('user', 'name'))", 'object_name': 'DashboardWorkspace'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_clonable': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_public': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'layout_uid': ('django.db.models.fields.CharField', [], {'max_length': '25'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'position': ('dash.fields.OrderField', [], {'null': 'True', 'blank': 'True'}),
            'slug': ('autoslug.fields.AutoSlugField', [], {'unique': 'True', 'max_length': '50', 'populate_from': "'name'", 'unique_with': '()'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm[user_orm_label]"})
        }
    }

    complete_apps = ['dash']



# -*- coding: utf-8 -*-
# Generated by Django 1.10.6 on 2017-03-11 20:49
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('dash', '0002_auto_20170126_1816'),
    ]

    operations = [
        migrations.AlterField(
            model_name='dashboardplugin',
            name='groups',
            field=models.ManyToManyField(blank=True, to='auth.Group', verbose_name='Group'),
        ),
        migrations.AlterField(
            model_name='dashboardplugin',
            name='users',
            field=models.ManyToManyField(blank=True, to=settings.AUTH_USER_MODEL, verbose_name='User'),
        ),
        migrations.AlterField(
            model_name='dashboardsettings',
            name='user',
            field=models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='User'),
        ),
    ]



from django import forms
from django.utils.translation import ugettext_lazy as _

from ....base import DashboardPluginFormBase
from ....widgets import BooleanRadioSelect

__title__ = 'dash.contrib.plugins.video.forms'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2017 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('VideoForm',)


class VideoForm(forms.Form, DashboardPluginFormBase):
    """Video form for ``VideoPlugin`` plugin."""

    plugin_data_fields = [
        ("title", ""),
        ("url", ""),
    ]

    title = forms.CharField(label=_("Title"), required=True)
    url = forms.URLField(label=_("URL"), required=True)



from django.db import models

__title__ = 'dash.fields'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2017 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'OrderField',
)


class OrderField(models.IntegerField):
    """OrderField.

    @author http://djangosnippets.org/users/zenx/
    @source http://djangosnippets.org/snippets/1861/

    OrderField for models from
    //ianonpython.blogspot.com/2008/08/orderfield-for-django-models.html and
    updated to use a django aggregation function. This field sets a default
    value as an auto-increment of the maximum value of the field +1.

    Ignores the incoming value and instead gets the maximum plus one of the
    field.

    This works really well in combination with "sortable_list.js". There are
    several things you should know:

        * order field shall be null=True, blank=True
        * order field shall not be unique

    If above mentioned is True, you can use jQuery drag-n-drop widget in
    your Django-admin.

    See the following example:

    >>> class Media:  # This belongs to your Admin class (admin.ModelAdmin)
    >>>     js = [
    >>>         '/media/js/jquery-1.6.2.min.js',
    >>>         '/media/js/jquery-ui-1.8.16.custom.min.js',
    >>>         '/media/js/sortable_list.js'
    >>>     ]
    """

    def pre_save(self, model_instance, value):
        # if the model is new and not an update
        if model_instance.pk is None:
            records = model_instance.__class__.objects.aggregate(
                models.Max(self.name)
            )
            if records:
                # get the maximum attribute from the first record and add
                # 1 to it
                try:
                    value = records['{0}__max'.format(self.name)] + 1
                except TypeError:
                    value = 1
            else:
                value = 1
        # otherwise the model is updating, pass the attribute value through
        else:
            value = getattr(model_instance, self.attname)
        return value

    # prevent the field from being displayed in the admin interface
    def formfield_(self, **kwargs):
        return None


# For south
try:
    from south.modelsinspector import add_introspection_rules
    rules = [
        (
            (OrderField,),
            [],
            {},  # takes widget only, which is standard
        ),
    ]

    add_introspection_rules(rules, ["^dash\.fields"])
except ImportError:
    pass



from ....contrib.plugins.image.dash_widgets import BaseImageWidget
from ....contrib.plugins.url.dash_widgets import (
    BaseBookmarkWidget,
    BaseURLWidget,
)

__title__ = 'dash.contrib.layouts.bootstrap3.dash_widgets'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2017 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'BaseBookmarkBootstrapThreeWidget',
    'BaseImageBootstrapThreeWidget',
    'BaseURLBootstrapThreeBootstrap3DashboardMainWidget',
)

# *************************************************************
# ******************* URL widgets *****************************
# *************************************************************


class BaseURLBootstrapThreeBootstrap3DashboardMainWidget(BaseURLWidget):
    """Base URL plugin widget for Bootstrap 3 Fluid layout.

    Placeholder `main` widget.
    """

    # layout_uid = 'bootstrap3_fluid'
    # placeholder_uid = 'main'
    # plugin_uid = 'url_bootstrap_three'
    media_css = (
        'bootstrap3/css/dash_plugin_url_bootstrap3.css',
    )

# *********************************************************
# *********************************************************
# *********************** Bookmark widgets ****************
# *********************************************************
# *********************************************************


class BaseBookmarkBootstrapThreeWidget(BaseBookmarkWidget):
    """Base Bookmark plugin widget for Bootstrap 3 Dashboard layout."""

    media_css = (
        'bootstrap3/css/dash_plugin_bookmark_bootstrap3.css',
    )

# *********************************************************
# *********************************************************
# *********************** Image widgets *******************
# *********************************************************
# *********************************************************


class BaseImageBootstrapThreeWidget(BaseImageWidget):
    """Base Image plugin widget for Bootstrap 3 Dashboard layout."""

    media_js = (
        'bootstrap3/js/dash_plugin_image_bootstrap3.js',
    )



from django.utils.translation import ugettext_lazy as _

from ....base import BaseDashboardPlugin
from ....factory import plugin_factory
from .forms import DummyForm, DummyShortcutsForm

__title__ = 'dash.contrib.plugins.dummy.dash_plugins'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2017 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('BaseDummyPlugin',)

# ****************************************************************************
# ***************************** Base  Dummy plugin ***************************
# ****************************************************************************


class BaseDummyPlugin(BaseDashboardPlugin):
    """Base dummy plugin."""

    name = _("Dummy")
    form = DummyForm
    group = _("Dummy")

    def get_form(self):
        """Get form."""
        if 'sidebar' == self.placeholder:
            return DummyShortcutsForm
        else:
            return DummyForm

    def post_processor(self):
        """If no text available, use dummy."""
        if not self.data.text:
            self.data.text = 'Dummy content'


# ****************************************************************************
# ********** Generating and registering the plugins using factory ************
# ****************************************************************************

sizes = (
    (1, 1),
    (1, 2),
    (2, 1),
    (2, 2),
    (3, 3),
    (4, 4),
    (5, 5)
)

plugin_factory(BaseDummyPlugin, 'dummy', sizes)



from django.core.exceptions import PermissionDenied
from django.contrib.auth.decorators import user_passes_test

__title__ = 'dash.decorators'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2017 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'all_permissions_required',
    'any_permission_required',
    'DEFAULT_SATISFY',
    'edit_dashboard_permission_required',
    'permissions_required',
    'SATISFY_ALL',
    'SATISFY_ANY',
)

SATISFY_ANY = 'any'
SATISFY_ALL = 'all'
DEFAULT_SATISFY = SATISFY_ALL


def permissions_required(perms, satisfy=DEFAULT_SATISFY, login_url=None,
                         raise_exception=False):
    """Check for the permissions given based on the strategy chosen.

    :param iterable perms:
    :param string satisfy: Allowed values are "all" and "any".
    :param string login_url:
    :param bool raise_exception: If set to True, the ``PermissionDenied``
        exception is raised on failures.
    :return bool:

    :example:
    >>> @login_required
    >>> @permissions_required(satisfy='any', perms=[
    >>>     'dash.add_dashboardentry',
    >>>     'dash.change_dashboardentry',
    >>>     'dash.delete_dashboardentry',
    >>>     'dash.add_dashboardworkspace',
    >>>     'dash.change_dashboardworkspace',
    >>>     'dash.delete_dashboardworkspace',
    >>>     'dash.add_dashboardsettings',
    >>>     'dash.change_dashboardsettings',
    >>>     'dash.delete_dashboardsettings',
    >>> ])
    >>> def edit_dashboard(request):
    >>>     # your code
    """
    assert satisfy in (SATISFY_ANY, SATISFY_ALL)

    if SATISFY_ALL == satisfy:
        # ``SATISFY_ALL`` case
        def check_perms(user):
            # First check if the user has the permission (even anon users)
            if user.has_perms(perms):
                return True
            # In case the 403 handler should be called raise the exception
            if raise_exception:
                raise PermissionDenied
            # As the last resort, show the login form
            return False
    else:
        # ``SATISFY_ANY`` case
        def check_perms(user):
            # First check if the user has the permission (even anon users)
            for perm in perms:
                if user.has_perm(perm):
                    return True

            # In case the 403 handler should be called raise the exception
            if raise_exception:
                raise PermissionDenied
            # As the last resort, show the login form
            return False

    return user_passes_test(check_perms, login_url=login_url)


def all_permissions_required(perms, login_url=None, raise_exception=False):
    """All permissions required.

    :example:
    >>> @login_required
    >>> @all_permissions_required([
    >>>     'dash.add_dashboardentry',
    >>>     'dash.change_dashboardentry',
    >>>     'dash.delete_dashboardentry',
    >>>     'dash.add_dashboardworkspace',
    >>>     'dash.change_dashboardworkspace',
    >>>     'dash.delete_dashboardworkspace',
    >>>     'dash.add_dashboardsettings',
    >>>     'dash.change_dashboardsettings',
    >>>     'dash.delete_dashboardsettings',
    >>> ])
    >>> def edit_dashboard(request):
    >>>     # your code
    """
    return permissions_required(
        perms,
        satisfy=SATISFY_ALL,
        login_url=login_url,
        raise_exception=raise_exception
    )


def any_permission_required(perms, login_url=None, raise_exception=False):
    """Any permission required.

    :example:
    >>> @login_required
    >>> @any_permission_required([
    >>>     'dash.add_dashboardentry',
    >>>     'dash.change_dashboardentry',
    >>>     'dash.delete_dashboardentry',
    >>>     'dash.add_dashboardworkspace',
    >>>     'dash.change_dashboardworkspace',
    >>>     'dash.delete_dashboardworkspace',
    >>>     'dash.add_dashboardsettings',
    >>>     'dash.change_dashboardsettings',
    >>>     'dash.delete_dashboardsettings',
    >>> ])
    >>> def edit_dashboard(request):
    >>>     # your code
    """
    return permissions_required(
        perms,
        satisfy=SATISFY_ANY,
        login_url=login_url,
        raise_exception=raise_exception
    )


def edit_dashboard_permission_required(login_url=None, raise_exception=False):
    """Check if user has permissions to edit dashboard.

    Simply, check is successful if any of the following permission checks are
    satisfied:

        - Can add dashboard entry
        - Can change dashboard entry
        - Can delete dashboard entry
        - Can add dashboard workspace
        - Can change dashboard workspace
        - Can delete dashboard workspace
        - Can add dashboard settings
        - Can change dashboard settings
        - Can delete dashboard settings

    :example:
    >>> @login_required
    >>> @edit_dashboard_permission_required() # Do not forget the brackets!
    >>> def edit_dashboard(request):
    >>>     # your code
    """
    return permissions_required(
        perms=[
            'dash.add_dashboardentry',
            'dash.change_dashboardentry',
            'dash.delete_dashboardentry',

            'dash.add_dashboardworkspace',
            'dash.change_dashboardworkspace',
            'dash.delete_dashboardworkspace',

            'dash.add_dashboardsettings',
            'dash.change_dashboardsettings',
            'dash.delete_dashboardsettings',
        ],
        satisfy=SATISFY_ANY,
        login_url=login_url,
        raise_exception=raise_exception
    )


def use_clipboard_permission_required(login_url=None, raise_exception=False):
    """Check if user has permissions to use clipboard.

    Simply, check is successful if all of the following permission checks are
    satisfied:

        - Can add dashboard entry
        - Can delete dashboard entry

    :example:
    >>> @login_required
    >>> @use_clipboard_permission_required() # Do not forget the brackets!
    >>> def cut_dashboard_entry(request):
    >>>     # your code
    """
    return permissions_required(
        perms=[
            'dash.add_dashboardentry',
            'dash.delete_dashboardentry'
        ],
        satisfy=SATISFY_ALL,
        login_url=login_url,
        raise_exception=raise_exception
    )



JSON_PACKAGE_USED = None
# Experimental switch to ujson
try:
    import ujson as json
    JSON_PACKAGE_USED = 'ujson'
except ImportError as e:
    try:
        import simplejson as json
        JSON_PACKAGE_USED = 'simplejson'
    except ImportError as e:
        import json
        JSON_PACKAGE_USED = 'json'

__title__ = 'dash.json_package'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2017 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'json',
    'JSON_PACKAGE_USED',
)



"""
- ``FIT_METHOD_CROP_SMART`` (string)
- ``FIT_METHOD_CROP_CENTER`` (string)
- ``FIT_METHOD_CROP_SCALE`` (string)
- ``FIT_METHOD_FIT_WIDTH`` (string)
- ``FIT_METHOD_FIT_HEIGHT`` (string)
- ``DEFAULT_FIT_METHOD`` (string)
- ``FIT_METHODS_CHOICES`` (tuple)
- ``FIT_METHODS_CHOICES_WITH_EMPTY_OPTION`` (list)
- ``IMAGES_UPLOAD_DIR`` (string)
"""
from .conf import get_setting

__title__ = 'dash.contrib.plugins.image.settings'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2017 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'DEFAULT_FIT_METHOD',
    'FIT_METHOD_CROP_CENTER',
    'FIT_METHOD_CROP_SCALE',
    'FIT_METHOD_CROP_SMART',
    'FIT_METHOD_FIT_HEIGHT',
    'FIT_METHOD_FIT_WIDTH',
    'FIT_METHODS_CHOICES',
    'FIT_METHODS_CHOICES_WITH_EMPTY_OPTION',
    'IMAGES_UPLOAD_DIR',
)

FIT_METHOD_CROP_SMART = get_setting('FIT_METHOD_CROP_SMART')
FIT_METHOD_CROP_CENTER = get_setting('FIT_METHOD_CROP_CENTER')
FIT_METHOD_CROP_SCALE = get_setting('FIT_METHOD_CROP_SCALE')
FIT_METHOD_FIT_WIDTH = get_setting('FIT_METHOD_FIT_WIDTH')
FIT_METHOD_FIT_HEIGHT = get_setting('FIT_METHOD_FIT_HEIGHT')
DEFAULT_FIT_METHOD = get_setting('DEFAULT_FIT_METHOD')
FIT_METHODS_CHOICES = get_setting('FIT_METHODS_CHOICES')
FIT_METHODS_CHOICES_WITH_EMPTY_OPTION = get_setting(
    'FIT_METHODS_CHOICES_WITH_EMPTY_OPTION'
)
IMAGES_UPLOAD_DIR = get_setting('IMAGES_UPLOAD_DIR')



import os

from django.conf import settings

DEFAULT_CONFIG = getattr(
    settings,
    'TINYMCE_DEFAULT_CONFIG',
    {'theme': "simple", 'relative_urls': False}
)

USE_SPELLCHECKER = getattr(settings, 'TINYMCE_SPELLCHECKER', False)

USE_COMPRESSOR = getattr(settings, 'TINYMCE_COMPRESSOR', False)

USE_FILEBROWSER = getattr(
    settings,
    'TINYMCE_FILEBROWSER',
    'filebrowser' in settings.INSTALLED_APPS
)

if 'staticfiles' in settings.INSTALLED_APPS \
        or 'django.contrib.staticfiles' in settings.INSTALLED_APPS:

    JS_URL = getattr(
        settings,
        'TINYMCE_JS_URL',
        os.path.join(settings.STATIC_URL, 'tiny_mce/tiny_mce.js')
    )
    JS_ROOT = getattr(
        settings,
        'TINYMCE_JS_ROOT',
        os.path.join(settings.STATIC_ROOT, 'tiny_mce')
    )

else:
    JS_URL = getattr(
        settings,
        'TINYMCE_JS_URL',
        '%sjs/tiny_mce/tiny_mce.js' % settings.MEDIA_URL
    )
    JS_ROOT = getattr(
        settings,
        'TINYMCE_JS_ROOT',
        os.path.join(settings.MEDIA_ROOT, 'js/tiny_mce')
    )

JS_BASE_URL = JS_URL[:JS_URL.rfind('/')]



from django.conf.urls import url

from .views import public_dashboard

__title__ = 'dash.contrib.apps.public_dashboard.urls'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2017 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('urlpatterns',)


urlpatterns = [
    # View public dashboard workspace.
    url(r'^(?P<username>[\w_\-]+)/(?P<workspace>[\w_\-]+)/$',
        view=public_dashboard,
        name='dash.public_dashboard'),

    # View public dashboard (no workspace selected == default workspace used).
    url(r'^(?P<username>[\w_\-]+)/$',
        view=public_dashboard,
        name='dash.public_dashboard'),
]



from ....base import (
    BaseDashboardLayout,
    BaseDashboardPlaceholder,
    layout_registry,
)

__title__ = 'dash.contrib.layouts.bootstrap2.dash_layouts'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2017 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('Bootstrap2FluidLayout',)

# *******************************************************************
# ******************** Bootstrap 2 Fluid layout *********************
# *******************************************************************


class Bootstrap2FluidMainPlaceholder(BaseDashboardPlaceholder):
    """Main placeholder."""

    uid = 'main'
    cols = 11
    rows = 9
    cell_width = 70
    cell_height = 40
    cell_margin_top = 8
    cell_margin_right = 8
    cell_margin_bottom = 8
    cell_margin_left = 8
    edit_template_name = 'bootstrap2/fluid_base_placeholder_edit.html'


class Bootstrap2FluidLayout(BaseDashboardLayout):
    """Bootstrap 2 Fluid layout."""

    uid = 'bootstrap2_fluid'
    name = 'Bootstrap 2 Fluid'
    view_template_name = 'bootstrap2/fluid_view_layout.html'
    edit_template_name = 'bootstrap2/fluid_edit_layout.html'
    plugin_widgets_template_name_ajax = 'bootstrap2/plugin_widgets_ajax.html'
    form_snippet_template_name = \
        'bootstrap2/snippets/generic_form_snippet.html'
    placeholders = [Bootstrap2FluidMainPlaceholder]
    cell_units = 'px'
    media_css = (
        'bootstrap2/css/bootstrap.css',
        'bootstrap2/css/dash_layout_bootstap2_fluid.css',
        # 'css/dash_solid_borders.css',
    )
    media_js = (
        'bootstrap2/js/bootstrap.js',
        'bootstrap2/js/dash_layout_bootstap2_fluid.js',
    )

    def get_view_template_name(self, request=None, origin=None):
        """Override the master view template for public dashboard app."""
        if origin == 'dash.public_dashboard':
            return 'bootstrap2/fuild_public_dashboard_view_layout.html'
        else:
            return super(Bootstrap2FluidLayout, self).get_view_template_name(
                request=request,
                origin=origin
            )


layout_registry.register(Bootstrap2FluidLayout)



from __future__ import absolute_import

from django.core.management.base import BaseCommand

from ...utils import sync_plugins

__title__ = 'dash.management.commands.dash_sync_plugins'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2017 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('Command',)


class Command(BaseCommand):
    """Adds the missing plugins to database (``dash.models.DashboardPlugin``).

    This command shall be ran every
    time a developer adds a new plugin.
    """

    def handle(self, *args, **options):

        sync_plugins()



import logging

from django.contrib.auth.decorators import login_required
from django.core.cache import cache
from django.core.exceptions import ValidationError
from django.core.validators import URLValidator
from django.template import RequestContext
from django.views.decorators.csrf import csrf_exempt
from django.views.decorators.http import require_POST

import feedparser

from nine import versions

from .defaults import (
    DEFAULT_MAX_FEED_ITEMS,
    DEFAULT_SHOW_TITLE,
    DEFAULT_TRUNCATE_AFTER,
)
from .forms import ReadRSSFeedForm
from .helpers import max_num_template

if versions.DJANGO_GTE_1_10:
    from django.shortcuts import render
else:
    from django.shortcuts import render_to_response

__title__ = 'dash.contrib.plugins.rss_feed.views'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2017 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('get_feed',)

logger = logging.getLogger(__name__)


@csrf_exempt
@require_POST
@login_required
def get_feed(request,
             layout_uid,
             placeholder_uid,
             plugin_uid,
             template_name='rss_feed/get_feed.html',
             template_name_ajax='rss_feed/get_feed_ajax.html'):
    """Get feed.

    :param django.http.HttpRequest request:
    :param str layout_uid:
    :param str placeholder_uid:
    :param str plugin_uid:
    :param str template_name:
    :param str template_name_ajax:
    :return django.http.HttpResponse:
    """
    form = ReadRSSFeedForm(request.POST)

    context = {}

    if form.is_valid():

        feed_url = form.cleaned_data.get('feed_url')
        custom_feed_title = form.cleaned_data.get('custom_feed_title', None)
        show_feed_title = form.cleaned_data.get('show_feed_title',
                                                DEFAULT_SHOW_TITLE)
        max_items = form.cleaned_data.get('max_items')
        truncate_after = form.cleaned_data.get('truncate_after',
                                               DEFAULT_TRUNCATE_AFTER)
        cache_for = form.cleaned_data.get('cache_for')

        key = '{0}-{1}-{2}-{3}'.format(layout_uid,
                                       placeholder_uid,
                                       plugin_uid,
                                       feed_url)
        feed = cache.get(key)

        if not feed:

            try:
                validate = URLValidator()
                valid_url = False
                try:
                    validate(feed_url)
                    valid_url = True
                except ValidationError as err:
                    valid_url = False
                    if DEBUG:
                        logger.debug(err)

                # Reading the feed and sending results to the template
                if valid_url:
                    # Saving raw data in case someone wants to show things
                    # differently.
                    feed = feedparser.parse(feed_url)
                    cache.set(key, feed, int(cache_for))

            except Exception as err:
                if DEBUG:
                    logger.debug(err)

        context = {
            'feed': feed,
            'custom_feed_title': custom_feed_title,
            'show_feed_title': show_feed_title,
            'truncate_after': truncate_after,
            'max_feed_items': max_num_template(max_items,
                                               DEFAULT_MAX_FEED_ITEMS)
        }

    if request.is_ajax():
        template_name = template_name_ajax

    if versions.DJANGO_GTE_1_10:
        return render(request, template_name, context)
    else:
        return render_to_response(
            template_name, context, context_instance=RequestContext(request)
        )



from django.template.loader import render_to_string

from ....base import BaseDashboardPluginWidget

__title__ = 'dash.contrib.plugins.memo.dash_widgets'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2017 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'BaseMemoWidget',
    'Memo1x1Widget',
    'Memo2x2Widget',
    'Memo3x3Widget',
    'Memo4x5Widget',
    'Memo5x5Widget',
    'Memo6x6Widget',
    'TinyMCEMemo2x2Widget',
    'TinyMCEMemo3x3Widget',
    'TinyMCEMemo4x4Widget',
    'TinyMCEMemo5x5Widget',
)

# ***********************************************************************
# ********************** Base widget for Memo plugin ********************
# ***********************************************************************


class BaseMemoWidget(BaseDashboardPluginWidget):
    """Base memo plugin widget."""

    def render(self, request=None):
        """Render."""
        context = {'plugin': self.plugin}
        return render_to_string('memo/render.html', context)

# ***********************************************************************
# ********************** Specific widgets for Memo plugin ***************
# ***********************************************************************


class Memo1x1Widget(BaseMemoWidget):
    """Memo 1x1 plugin widget."""

    plugin_uid = 'memo_1x1'
    cols = 1
    rows = 1


class Memo2x2Widget(BaseMemoWidget):
    """Memo 2x2 plugin widget."""

    plugin_uid = 'memo_2x2'
    cols = 2
    rows = 2


class Memo3x3Widget(BaseMemoWidget):
    """Memo 3x3 plugin widget."""

    plugin_uid = 'memo_3x3'
    cols = 3
    rows = 3


class Memo4x5Widget(BaseMemoWidget):
    """Memo 4x5 plugin widget."""

    plugin_uid = 'memo_4x5'
    cols = 4
    rows = 5


class Memo5x5Widget(BaseMemoWidget):
    """Memo 5x5 plugin widget."""

    plugin_uid = 'memo_5x5'
    cols = 5
    rows = 5


class Memo6x6Widget(BaseMemoWidget):
    """Memo 6x6 plugin widget."""

    plugin_uid = 'memo_6x6'
    cols = 6
    rows = 6

# ***********************************************************************
# ********************** Base widget for Memo plugin ********************
# ***********************************************************************


class BaseTinyMCEMemoWidget(BaseDashboardPluginWidget):
    """Base TinyMCE memo plugin widget."""

    def render(self, request=None):
        """Render."""
        context = {'plugin': self.plugin}
        return render_to_string('tinymce/render.html', context)

# ***********************************************************************
# ****************** Specific widgets for TinyMCEMemo plugin ************
# ***********************************************************************


class TinyMCEMemo2x2Widget(BaseTinyMCEMemoWidget):
    """TinyMCE memo 2x2 plugin widget."""

    plugin_uid = 'tinymce_memo_2x2'
    cols = 2
    rows = 2


class TinyMCEMemo3x3Widget(BaseTinyMCEMemoWidget):
    """TinyMCE memo 3x3 plugin widget."""

    plugin_uid = 'tinymce_memo_3x3'
    cols = 3
    rows = 3


class TinyMCEMemo4x4Widget(BaseTinyMCEMemoWidget):
    """TinyMCE memo 4x4 plugin widget."""

    plugin_uid = 'tinymce_memo_4x4'
    cols = 4
    rows = 4


class TinyMCEMemo5x5Widget(BaseTinyMCEMemoWidget):
    """TinyMCE memo 5x5 plugin widget."""

    plugin_uid = 'tinymce_memo_5x5'
    cols = 5
    rows = 5



from django import forms
from django.conf.urls import url
from django.contrib import admin
from django.contrib import messages
from django.contrib.admin import helpers
from django.contrib.admin.views.decorators import staff_member_required
from django.shortcuts import redirect
from django.template import RequestContext
from django.utils.decorators import method_decorator
from django.utils.translation import ugettext_lazy as _

from nine.versions import DJANGO_LTE_1_5, DJANGO_GTE_1_10

from .base import get_registered_plugins, get_registered_layouts
from .constants import ACTION_CHOICE_REPLACE
from .models import (
    DashboardEntry,
    DashboardPlugin,
    DashboardSettings,
    DashboardWorkspace,
)
from .forms import BulkChangeDashboardPluginsForm

if DJANGO_GTE_1_10:
    from django.shortcuts import render
else:
    from django.shortcuts import render_to_response

staff_member_required_m = method_decorator(staff_member_required)

__title__ = 'dash.admin'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2017 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'bulk_change_dashboard_plugins',
    'DashboardEntryAdmin',
    'DashboardPluginAdmin',
    'DashboardSettingsAdmin',
    'DashboardWorkspaceAdmin',
)

# *********************************************************
# ************************ Admin helpers ******************
# *********************************************************


def bulk_change_dashboard_plugins(modeladmin, request, queryset):
    """Bulk change of dashboard plugins action additional view.

    Data is changed in ``DashboardPluginAdmin.bulk_change_dashboard_plugins``
    method.
    """
    opts = modeladmin.model._meta
    app_label = opts.app_label

    selected = request.POST.getlist(admin.ACTION_CHECKBOX_NAME)
    post = dict(request.POST)
    if selected:
        post['selected_dashboard_plugins'] = ','.join(selected)
    if 'POST' == request.method:
        form = BulkChangeDashboardPluginsForm(
            data=post,
            files=request.FILES,
            initial={'selected_dashboard_plugins': ','.join(selected)}
        )
    else:
        form = BulkChangeDashboardPluginsForm(
            initial={'selected_dashboard_plugins': ','.join(selected)}
        )

    context = {
        'form': form,
        'app_label': app_label,
        'opts': opts,
        'action_checkbox_name': helpers.ACTION_CHECKBOX_NAME,
    }

    template_name = 'dash/admin/bulk_change_dashboard_plugins.html'

    if DJANGO_GTE_1_10:
        return render(request, template_name, context)
    else:
        return render_to_response(
            template_name, context, context_instance=RequestContext(request)
        )


class CompatModelAdmin(admin.ModelAdmin):
    """Compatibility model admin."""

    def __init__(self, *args, **kwargs):
        self.queryset = self.get_queryset
        super(CompatModelAdmin, self).__init__(*args, **kwargs)

    def get_queryset(self, request):
        """Get queryset."""
        superobj = super(CompatModelAdmin, self)
        if getattr(superobj, 'get_queryset', None):
            return superobj.get_queryset(request)

        if getattr(superobj, 'queryset', None):
            return superobj.queryset(request)


class DashboardWorkspaceAdmin(admin.ModelAdmin):
    """Dashboard workspace admin."""

    list_display = ('name', 'slug', 'layout_uid', 'position', 'user',
                    'is_public')
    list_editable = ('position',)
    list_filter = ('layout_uid', 'is_public')
    readonly_fields = ('slug',)
    fieldsets = (
        (None, {
            'fields': ('name', 'position', 'is_public')
        }),
        (_("User"), {
            'fields': ('user',)
        }),
        (_('Additional'), {
            'classes': ('collapse',),
            'fields': ('slug',)
        }),
    )

    class Meta(object):
        """Meta."""

        app_label = _('Dashboard workspace')


admin.site.register(DashboardWorkspace, DashboardWorkspaceAdmin)


class DashboardEntryAdminForm(forms.ModelForm):
    """Dashboard entry admin form."""

    class Meta(object):
        """Meta."""

        model = DashboardEntry
        exclude = []

    layout_uid = forms.ChoiceField(choices=get_registered_layouts())
    plugin_uid = forms.ChoiceField(choices=get_registered_plugins())


class DashboardEntryAdmin(CompatModelAdmin):
    """Dashboard entry admin."""
    form = DashboardEntryAdminForm
    list_display = ('plugin_uid', 'plugin_uid_code', 'plugin_data',
                    'layout_uid', 'placeholder_uid', 'position',
                    'workspace', 'user')
    list_filter = ('user', 'workspace', 'layout_uid', 'placeholder_uid',
                   'plugin_uid')
    list_editable = ('position',)
    readonly_fields = ('plugin_uid_code',)
    fieldsets = (
        (None, {
            'fields': ('plugin_uid', 'plugin_data', 'layout_uid',
                       'placeholder_uid', 'position', 'workspace')
        }),
        (_("User"), {
            'fields': ('user',)
        }),
    )

    class Meta(object):
        """Meta."""

        app_label = _('Dashboard entry')

    def __queryset(self, request):
        if DJANGO_LTE_1_5:
            queryset = super(DashboardEntryAdmin, self).queryset(request)
        else:
            queryset = super(DashboardEntryAdmin, self).get_queryset(request)
        queryset = queryset.select_related('workspace', 'user')
        return queryset

    get_queryset = __queryset
    if DJANGO_LTE_1_5:
        queryset = __queryset


admin.site.register(DashboardEntry, DashboardEntryAdmin)


class DashboardPluginAdminForm(forms.ModelForm):
    """Dashboard plugin admin form."""

    class Meta(object):
        """Meta."""

        model = DashboardPlugin
        exclude = []

    plugin_uid = forms.ChoiceField(choices=get_registered_plugins())


class DashboardPluginAdmin(CompatModelAdmin):
    """Dashboard plugin admin."""

    form = DashboardPluginAdminForm
    list_display = ('plugin_uid_admin', 'users_list', 'groups_list')
    readonly_fields = ('plugin_uid', 'plugin_uid_admin')
    fieldsets = (
        (None, {
            'fields': ('plugin_uid', 'users', 'groups')
        }),
    )
    filter_horizontal = ('users', 'groups',)
    actions = [bulk_change_dashboard_plugins]

    class Meta(object):
        """Meta."""

        app_label = _('Dashboard plugin')

    def __queryset(self, request):
        if DJANGO_LTE_1_5:
            queryset = super(DashboardPluginAdmin, self).queryset(request)
        else:
            queryset = super(DashboardPluginAdmin, self).get_queryset(request)

        queryset = queryset.prefetch_related('users', 'groups')
        return queryset

    get_queryset = __queryset
    if DJANGO_LTE_1_5:
        queryset = __queryset

    @staff_member_required_m
    def bulk_change_dashboard_plugins(self, request):
        """This is where the data is actually processed."""
        if 'POST' == request.method:
            form_cls = BulkChangeDashboardPluginsForm
            form = form_cls(
                data=request.POST,
                files=request.FILES
            )
            if form.is_valid():
                ids = form.cleaned_data.pop(
                    'selected_dashboard_plugins'
                ).split(',')
                users = form.cleaned_data.pop('users')
                groups = form.cleaned_data.pop('groups')
                users_action = form.cleaned_data.pop('users_action')
                groups_action = form.cleaned_data.pop('groups_action')
                cleaned_data = dict(
                    (key, val) for (key, val)
                    in form.cleaned_data.iteritems()
                    if val is not None
                )

                # Queryset to work with
                queryset = DashboardPlugin._default_manager.filter(pk__in=ids)

                # Update simple fields
                updated = queryset.update(**cleaned_data)

                # Update groups
                for dashboard_plugin in queryset:
                    # If groups action chose is ``replace``, clearing the
                    # groups first.
                    if groups_action == ACTION_CHOICE_REPLACE:
                        dashboard_plugin.groups.clear()

                    # If users action chose is ``replace``, clearing the
                    # users first.
                    if users_action == ACTION_CHOICE_REPLACE:
                        dashboard_plugin.users.clear()

                    dashboard_plugin.groups.add(*groups)  # Adding groups
                    dashboard_plugin.users.add(*users)  # Adding users

                messages.info(
                    request,
                    _('{0} Dashboard plugins were '
                      'changed successfully.').format(len(ids))
                )

                return redirect('admin:dash_dashboardplugin_changelist')
        else:
            messages.warning(
                request,
                _('POST required when changing in bulk!')
            )
            return redirect('admin:dash_dashboardplugin_changelist')

    def get_urls(self):
        """Get urls."""
        my_urls = [
            # Bulk change dashboard plugins
            url(r'^bulk-change-dashboard-plugins/$',
                self.bulk_change_dashboard_plugins,
                name='bulk_change_dashboard_plugins'),
        ]
        return my_urls + super(DashboardPluginAdmin, self).get_urls()


admin.site.register(DashboardPlugin, DashboardPluginAdmin)


class DashboardSettingsAdminForm(forms.ModelForm):
    """Dashboard settings admin form."""

    class Meta(object):
        """Meta."""

        model = DashboardSettings
        exclude = []

    layout_uid = forms.ChoiceField(choices=get_registered_layouts())


class DashboardSettingsAdmin(CompatModelAdmin):
    """Dashboard settings admin."""

    form = DashboardSettingsAdminForm
    list_display = ('title', 'user', 'layout_uid', 'is_public')
    # readonly_fields = ('plugin_uid',)
    fieldsets = (
        (None, {
            'fields': ('title', 'user', 'layout_uid', 'is_public')
        }),
    )

    class Meta(object):
        """Meta."""
        app_label = _('Dashboard settings')

    def __queryset(self, request):
        if DJANGO_LTE_1_5:
            queryset = super(DashboardSettingsAdmin, self).queryset(request)
        else:
            queryset = \
                super(DashboardSettingsAdmin, self).get_queryset(request)

        queryset = queryset.select_related('user')
        return queryset

    get_queryset = __queryset
    if DJANGO_LTE_1_5:
        queryset = __queryset


admin.site.register(DashboardSettings, DashboardSettingsAdmin)



import logging

from six import PY2

from django import forms
from django.utils.text import Truncator
from django.utils.translation import ugettext_lazy as _

from transliterate.contrib.apps.translipsum import TranslipsumGenerator

from ....base import DashboardPluginFormBase
from ....widgets import BooleanRadioSelect
from .defaults import (
    DEFAULT_MAX_CHARS,
    LANGUAGE_CHOICES,
    LANGUAGE_CHOICES_KEYS,
)

if PY2:
    from lipsum import Generator
else:
    from transliterate.contrib.apps.translipsum.utils import Generator


__title__ = 'dash.contrib.plugins.dummy.forms'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2017 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('DummyForm', 'DummyShortcutsForm')

logger = logging.getLogger(__name__)


class DummyForm(forms.Form, DashboardPluginFormBase):
    """Dummy form (for main `placeholder`)."""

    plugin_data_fields = [
        ("show_title", False),
        ("generate_lipsum", False),
        ("lipsum_language", ""),
        ("lipsum_max_chars", DEFAULT_MAX_CHARS),
        ("text", "")
    ]
    show_title = forms.BooleanField(label=_("Show title?"),
                                    required=False,
                                    initial=False,
                                    widget=BooleanRadioSelect)
    generate_lipsum = forms.BooleanField(label=_("Generate lorem ipsum?"),
                                         required=False,
                                         initial=False,
                                         widget=BooleanRadioSelect)
    lipsum_language = forms.ChoiceField(label=_("Language"),
                                        required=False,
                                        choices=LANGUAGE_CHOICES)
    lipsum_max_chars = forms.IntegerField(label=_("Max number of chars for "
                                                  "generated text"),
                                          required=True,
                                          initial=DEFAULT_MAX_CHARS)
    text = forms.CharField(label=_("Generated lorem ipsum text"),
                           required=False,
                           widget=forms.widgets.HiddenInput)

    def save_plugin_data(self, request=None):
        """Save plugin data.

        We want to save the generated lorem ipsum text for later use.
        Thus, although we don't show it to the user, in case when
        ``generate_lipsum`` field is set to True, we silently generate the
        text and save it into the plugin data.
        """
        if self.cleaned_data.get('generate_lipsum', None):
            lipsum_language = self.cleaned_data.get('lipsum_language', None)
            try:
                if lipsum_language in LANGUAGE_CHOICES_KEYS:
                    if lipsum_language == 'en':
                        gen = Generator()
                    else:
                        gen = TranslipsumGenerator(
                            language_code=lipsum_language
                        )
                    text = gen.generate_paragraph()
                    truncator = Truncator(text)
                    self.cleaned_data['text'] = truncator.chars(
                        self.cleaned_data.get('lipsum_max_chars',
                                              DEFAULT_MAX_CHARS)
                    )
            except Exception as err:
                if DEBUG:
                    logger.debug(err)


class DummyShortcutsForm(DummyForm):
    """Dummy form for `shortucts` placeholder."""



try:
    from django.apps import AppConfig

    class Config(AppConfig):
        """Config."""

        name = 'dash.contrib.apps.public_dashboard'
        label = 'dash_contrib_apps_public_dashboard'

except ImportError:
    pass

__title__ = 'dash.contrib.apps.public_dashboard.apps'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2017 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('Config',)



__title__ = 'dash.contrib.plugins.weather.defaults'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2017 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'API_ENDPOINT_URL',
    'API_KEY',
    'DEFAULT_CACHE_FOR',
    'DEFAULT_SHOW_TITLE',
)

# http://developer.worldweatheronline.com API key.
API_KEY = ''

# Endpoint URL for the weather API. Should get (key, format, q) arguments.
API_ENDPOINT_URL = 'http://api.worldweatheronline.com/free/v1/weather.ashx' \
                   '?key={0}&format={1}&q={2}'

DEFAULT_SHOW_TITLE = True

DEFAULT_CACHE_FOR = 3600



from django import forms
from django.utils.translation import ugettext_lazy as _

from ....base import DashboardPluginFormBase

try:
    from tinymce.widgets import TinyMCE
except ImportError:
    from ....lib.tinymce.widgets import TinyMCE

__title__ = 'dash.contrib.plugins.memo.forms'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2017 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'MCE_ATTRS',
    'MemoForm',
    'TinyMCEMemoForm',
)


class MemoForm(forms.Form, DashboardPluginFormBase):
    """Memo form (for ``Memo`` plugin)."""

    plugin_data_fields = [
        ("title", ""),
        ("text", "")
    ]

    title = forms.CharField(label=_("Title"), required=False)
    text = forms.CharField(label=_("Text"), required=True,
                           widget=forms.widgets.Textarea)


# Basic TinyMCE config
MCE_ATTRS = {
    'plugins': 'visualchars,paste',
    'theme': 'advanced',
    'theme_advanced_buttons1':
        'formatselect,|,bold,italic,underline,|,bullist,numlist',
    'theme_advanced_buttons2': 'link,unlink,|,code',
    'width': '300',
    'delta_height': '150',
    'relative_urls': 0
}


class TinyMCEMemoForm(forms.Form, DashboardPluginFormBase):
    """TinyMCE memo form (for ``TinyMCEMemo`` plugin)."""

    plugin_data_fields = [
        ("title", ""),
        ("text", "")
    ]

    title = forms.CharField(label=_("Title"), required=False)
    text = forms.CharField(label=_("HTML"), required=True,
                           widget=TinyMCE(mce_attrs=MCE_ATTRS))



"""
- `RESTRICT_PLUGIN_ACCESS` (bool): If set to True, (Django) permission system
  for dash plugins is enabled.
- `PLUGINS_MODULE_NAME` (str): Name of the module to placed in the (external)
  apps in which the dash plugin code should be implemented and registered.
- `ACTIVE_LAYOUT` (str): Active layout UID.
- `LAYOUTS_MODULE_NAME` (str): Name of the python module to be placed in (
  external) apps in which the dash layouts should be implemented and
  registered.
- `DEFAULT_WORKSPACE_NAME` (str): Name of the default workspace.
- `DEFAULT_PLACEHOLDER_VIEW_TEMPLATE_NAME` (str): Default template name for
  the placeholder view.
- `DEFAULT_PLACEHOLDER_EDIT_TEMPLATE_NAME` (str): Default template name for
  the placeholder edit.
- `LAYOUT_CELL_UNITS` (str): Layout cell units. Allowed values are `em`, `px`,
  `pt`, `%`.
- `DISPLAY_AUTH_LINK` (bool): If set to True, the log in or log out link is
  shown in the Dash drop-down menu.
- `DEBUG` (bool)
"""
from .conf import get_setting

__title__ = 'dash.settings'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2017 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'ACTIVE_LAYOUT',
    'DEBUG',
    'DEFAULT_PLACEHOLDER_EDIT_TEMPLATE_NAME',
    'DEFAULT_PLACEHOLDER_VIEW_TEMPLATE_NAME',
    'DEFAULT_WORKSPACE_NAME',
    'DISPLAY_AUTH_LINK',
    'DISPLAY_LOGOUT_LINK',
    'LAYOUT_CELL_UNITS',
    'LAYOUTS_MODULE_NAME',
    'PLUGIN_CLIPBOARD_KEY',
    'PLUGINS_MODULE_NAME',
    'RAISE_EXCEPTION_WHEN_PERMISSIONS_INSUFFICIENT',
    'RESTRICT_PLUGIN_ACCESS',
    'WAIT_AT_TEST_END',
    'WAIT_BETWEEN_TEST_STEPS',
)

RESTRICT_PLUGIN_ACCESS = get_setting('RESTRICT_PLUGIN_ACCESS')

RAISE_EXCEPTION_WHEN_PERMISSIONS_INSUFFICIENT = get_setting(
    'RAISE_EXCEPTION_WHEN_PERMISSIONS_INSUFFICIENT'
)

PLUGINS_MODULE_NAME = get_setting('PLUGINS_MODULE_NAME')

ACTIVE_LAYOUT = get_setting('ACTIVE_LAYOUT')

LAYOUTS_MODULE_NAME = get_setting('LAYOUTS_MODULE_NAME')

DEFAULT_WORKSPACE_NAME = get_setting('DEFAULT_WORKSPACE_NAME')

DEFAULT_PLACEHOLDER_VIEW_TEMPLATE_NAME = get_setting(
    'DEFAULT_PLACEHOLDER_VIEW_TEMPLATE_NAME'
)

DEFAULT_PLACEHOLDER_EDIT_TEMPLATE_NAME = get_setting(
    'DEFAULT_PLACEHOLDER_EDIT_TEMPLATE_NAME'
)

LAYOUT_CELL_UNITS = get_setting('LAYOUT_CELL_UNITS')

DISPLAY_AUTH_LINK = get_setting('DISPLAY_AUTH_LINK')
DISPLAY_LOGOUT_LINK = DISPLAY_AUTH_LINK

WAIT_BETWEEN_TEST_STEPS = get_setting('WAIT_BETWEEN_TEST_STEPS')
WAIT_AT_TEST_END = get_setting('WAIT_AT_TEST_END')

PLUGIN_CLIPBOARD_KEY = get_setting('PLUGIN_CLIPBOARD_KEY')

DEBUG = get_setting('DEBUG')



__title__ = 'dash.contrib.plugins.rss_feed.helpers'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2017 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('max_num_template',)


def max_num_template(max_items, default):
    """Nax num of items in a template."""
    return ':{0}'.format(max_items if max_items else default)
